// ===========================================================================
//	CComparisonWindow.h
// ===========================================================================


#pragma once


#include <LWindow.h>
#include <LPeriodical.h>
#include <UGWorld.h>


const	ResIDT	WIND_Comparison = 		3000;
const	PaneIDT	paneID_Comparison = 3000;


class CComparisonWindow;
class CTinselTestWindow;

typedef void (CComparisonWindow::*PixelFunctionPtr)(Uint8*, Uint8*, Uint8*, Uint8*);


class	CComparisonWindow
	 : public LWindow,
		 public LPeriodical {
public:
	enum { class_ID = 'Cwnd' };

	enum ECompareMode {
		eCompare_Red,
		eCompare_Green,
		eCompare_Blue,
		eCompare_Alpha,
		eCompare_Average,
		eCompare_Left,
		eCompare_Right,
		eCompare_Alternate
	};
	
	enum EChannel {
		eChannel_Alpha = 0,
		eChannel_Red,
		eChannel_Green,
		eChannel_Blue
	};	
	
	enum kARGB16mask {
		kARGB16mask_Alpha = 	(0x8000),
		kARGB16mask_Red = 		(0x7C00),
		kARGB16mask_Green = 	(0x03E0),
		kARGB16mask_Blue =  	(0x001F)
	};

	enum kARGB16shift {
		kARGB16shift_Alpha = 	15,
		kARGB16shift_Red = 		10,
		kARGB16shift_Green = 	5,
		kARGB16shift_Blue =  	0
	};

	static CComparisonWindow*	CreateComparisonWindow(ResIDT inWindowID,
								LCommander *inSuperCommander);
	static CComparisonWindow*	CreateComparisonWindowStream(LStream *inStream);
	
	static void CompareWindows(
		CTinselTestWindow* inWindow1, 
		CTinselTestWindow* inWindow2);
	
	static void GetDefaultGlobalRects(
				GDHandle inDevice, 
				Rect* outLeftRect, 
				Rect* outRightRect);
	
		static CComparisonWindow*
	GetWindow()
	{
		return sWindow;
	}
	
	void InitComparisonWindow();
	
	void ListenToMessage(MessageT inMessage, void* ioParam);

	virtual Boolean			ObeyCommand(
								CommandT	inCommand,
								void		*ioParam = nil);
								
	virtual void FindCommandStatus(
		CommandT	inCommand,
		Boolean		&outEnabled,
		Boolean		&outUsesMark,
		Char16		&outMark,
		Str255		outName);
	
	void PutOnDuty();
	void TakeOffDuty();
	
//	Boolean AskSaveAs();
//	void DoAESave(
//		FSSpec&	inFileSpec,
//		OSType	inFileType);
//		
//	void Clear();
	
		LGWorld*
	GetGWorld()
	{
		return mGWorld;
	}
	
		ECompareMode
	GetCompareMode()
	{
		return mCompareMode;
	}
	
	void SetCompareMode(ECompareMode inCompareMode);
	

protected:
	static CComparisonWindow* sWindow;
	
	LGWorld* mGWorld;
	LGWorld* mComparisonGWorld;
	LGWorld* mChannelGWorld;
	ECompareMode mCompareMode;
	unsigned long mLastSwitchTime;
	Boolean mIsAlternating;

	CComparisonWindow();
	CComparisonWindow(const SWindowInfo &inWindowInfo);
	CComparisonWindow(ResIDT inWINDid, Uint32 inAttributes,
				LCommander *inSuperCommander);
	CComparisonWindow(LStream *inStream);
	~CComparisonWindow();

	void SpendTime(const EventRecord &inMacEvent);
	
		long
	PixMapRowBytes(const PixMapHandle inPixMapH)
	{
		return ((**(inPixMapH)).rowBytes & 0x3FFF);
	}
	
	void MapPixels(
		PixelFunctionPtr inFunction,
		GWorldPtr	inGWorld1,
		GWorldPtr	inGWorld2 = nil,
		GWorldPtr	inGWorld3 = nil);
	void CompareGWorlds (
		const GWorldPtr	inGWorld1,
		const GWorldPtr	inGWorld2);
	void ProcessChannel(PixelFunctionPtr inFunction);
	void ResizeTo(short inWidth, short inHeight);
	void ResizeAndCenter(const Rect& inRect);
	virtual void DrawSelf();
	LGWorld* CreateGWorld(Rect& inRect, Int16 inPixelDepth);
		
		void 
	DifferencePixel(Uint8* ioA, Uint8* ioR, Uint8* ioG, Uint8* ioB)
	{
			// for each channel, store (in the third GWorld) the inverse, 
			// absolute difference between the corresponding channels in the 
			// first two GWorld.  e.g., in a 32-bit GWorld, if both of the
			// alpha channels are the same, ioA[2] will get 255.
		ioA[2] = (ioA[0] > ioA[1]) ? ~(ioA[0] - ioA[1]) : ~(ioA[1] - ioA[0]);
		ioR[2] = (ioR[0] > ioR[1]) ? ~(ioR[0] - ioR[1]) : ~(ioR[1] - ioR[0]);
		ioG[2] = (ioG[0] > ioG[1]) ? ~(ioG[0] - ioG[1]) : ~(ioG[1] - ioG[0]);
		ioB[2] = (ioB[0] > ioB[1]) ? ~(ioB[0] - ioB[1]) : ~(ioB[1] - ioB[0]);
	}
		
		void 
	AveragePixel(Uint8* ioA, Uint8* ioR, Uint8* ioG, Uint8* ioB)
	{
		Uint32 average = (ioA[0] + ioR[0] + ioG[0] + ioB[0]) >> 2;
		ioA[1] = ioR[1] = ioG[1] = ioB[1] = average;
	}
		
		void 
	ExtractAlpha(Uint8* ioA, Uint8* ioR, Uint8* ioG, Uint8* ioB)
	{
		ioR[1] = ioG[1] = ioB[1] = ioA[0];
//		ioA[1] = ioR[1] = ioG[1] = ioB[1] = ioA[0];
	}
		
		void 
	ExtractRed(Uint8* ioA, Uint8* ioR, Uint8* ioG, Uint8* ioB)
	{
		ioA[1] = ioR[1] = ioG[1] = ioB[1] = ioR[0];
	}
		
		void 
	ExtractGreen(Uint8* ioA, Uint8* ioR, Uint8* ioG, Uint8* ioB)
	{
		ioA[1] = ioR[1] = ioG[1] = ioB[1] = ioG[0];
	}
		
		void 
	ExtractBlue(Uint8* ioA, Uint8* ioR, Uint8* ioG, Uint8* ioB)
	{
		ioA[1] = ioR[1] = ioG[1] = ioB[1] = ioB[0];
	}
};

