// ===========================================================================
// CTQAImage.h
// ===========================================================================


#pragma once


#include <QuickDraw.h>

#include "RAVE.h"


#define kMaxImageCount 20


class CTQAImage {
public:
	CTQAImage(
		PicHandle 					inPICTHandle,
		TQAImagePixelType 	inPixelType,
		Boolean							inMakeBlackTransparent = false,
		Boolean							inMakeMipMap = false);
	~CTQAImage();
	
		TQAImage*
	GetImages()
	{
		return mImages;
	}
	
		long
	GetWidth()
	{
		return mImages[0].width;
	}
	
		long
	GetHeight()
	{
		return mImages[0].height;
	}
	
		long
	GetRowBytes()
	{
		return mImages[0].rowBytes;
	}
	
		CTabHandle
	GetColorTableHandle()
	{
		return (**mGWorld->GetMacGWorld()->portPixMap).pmTable;
	}
	
		Boolean
	IsValidTexture()
	{
		return mIsValidTexture;
	}
	
		Boolean
	IsValidBitmap()
	{
		return mIsValidBitmap;
	}
	
		Boolean
	IsValidMipMap()
	{
		return mIsValidMipMap;
	}
	
protected:
	TQAImagePixelType 	mPixelType;
	TQAImage 						mImages[kMaxImageCount];
	LGWorld*						mGWorld;
	void*								mMipMaps;
	Boolean							mIsValidTexture;
	Boolean							mIsValidBitmap;
	Boolean							mIsValidMipMap;
	
	void MakeMipMap();
	
		long
	AlphaChannel(Uint8* inPixelP)
	{
		switch (mPixelType) {
			case kQAPixel_RGB32:
				return 255;
				break;
				
			case kQAPixel_RGB16:
				return 1;
				break;
				
			case kQAPixel_ARGB32:
				return inPixelP[0];
				break;
				
			case kQAPixel_ARGB16:
				return (inPixelP[0] & 0x80) >> 7;
				break;
		} // endswitch
		
		return 0;
	}
	
		long
	RedChannel(Uint8* inPixelP)
	{
		switch (mPixelType) {
			case kQAPixel_ARGB32:
			case kQAPixel_RGB32:
				return inPixelP[1];
				break;
				
			case kQAPixel_ARGB16:
			case kQAPixel_RGB16:
				return (inPixelP[0] & 0x7C) >> 2;
				break;
		} // endswitch
		
		return 0;
	}
	
		long
	GreenChannel(Uint8* inPixelP)
	{
		switch (mPixelType) {
			case kQAPixel_ARGB32:
			case kQAPixel_RGB32:
				return inPixelP[2];
				break;
				
			case kQAPixel_ARGB16:
			case kQAPixel_RGB16:
					// the green bits straddle the two bytes
				return ((inPixelP[0] & 0x03) << 3) + ((inPixelP[1] & 0xE0) >> 5);
				break;
		} // endswitch
		
		return 0;
	}
	
		long
	BlueChannel(Uint8* inPixelP)
	{
		switch (mPixelType) {
			case kQAPixel_ARGB32:
			case kQAPixel_RGB32:
				return inPixelP[3];
				break;
				
			case kQAPixel_ARGB16:
			case kQAPixel_RGB16:
				return (inPixelP[1] & 0x1F);
				break;
		} // endswitch
		
		return 0;
	}
	
		void
	SetARGB(Uint8* inPixelP, long inA, long inR, long inG, long inB)
	{
		switch (mPixelType) {
			case kQAPixel_ARGB32:
			case kQAPixel_RGB32:
				inPixelP[0] = (Uint8) inA;
				inPixelP[1] = (Uint8) inR;
				inPixelP[2] = (Uint8) inG;
				inPixelP[3] = (Uint8) inB;
				break;
				
			case kQAPixel_ARGB16:
			case kQAPixel_RGB16:
				inPixelP[0] = (Uint8)
					((inA & 1)    << 7) + 
					((inR & 0x1F) << 2) + 
					((inG & 0x18) >> 3);
				inPixelP[1] = (Uint8)
					((inG & 0x07) << 5) + 
					(inB & 0x1F);
				break;
		} // endswitch
	}
	
private:
	CTQAImage();
};
