/*
 	File:		MixedModePriv.h
 
 	Contains:	Private interfaces for the Mixed Mode mechanism
 
 	Version:	Maxwell
 
 	DRI:		Dave Peterson
 
 	Copyright:	 1984-1995 by Apple Computer, Inc.
 				All rights reserved.
 
 	Warning:	*** APPLE INTERNAL USE ONLY ***
 				This file contains unreleased SPI's
 
 	BuildInfo:	Built by:			Build God
 				With Interfacer:	1.1d12  
 				From:				MixedModePriv.i
 					Revision:		20
 					Dated:			12/12/94
 					Last change by:	CSS
 					Last comment:	This file doesn't work for Pascal or Assembly so pragma output
 
 	Bugs:		Report bugs to Radar component System Interfaces, Latest
 				List the version information (from above) in the Problem Description.
 
*/

#ifndef __MIXEDMODEPRIV__
#define __MIXEDMODEPRIV__


#ifndef __TYPES__
#include <Types.h>
#endif
/*	#include <ConditionalMacros.h>								*/

#ifndef __MIXEDMODE__
#include <MixedMode.h>
#endif

#ifndef __CODEFRAGMENTS__
#include <CodeFragments.h>
#endif
/*	#include <Files.h>											*/
/*		#include <OSUtils.h>									*/
/*			#include <Memory.h>									*/
/*	#include <Kernel.h>											*/
/*		#include <Errors.h>										*/
/*		#include <MachineExceptions.h>							*/
/*		#include <Timing.h>										*/
/*	#include <FileSystemsTypes.h>								*/

#ifndef __NANOKERNELPRIV__
#include <NanoKernelPriv.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if STRUCTALIGNMENTSUPPORTED
#pragma options align=mac68k
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import on
#endif

#define OSTrapTableAddress(trapNum) (((trapNum & 0x00FF) << 2) + 0x400)
#define TBTrapTableAddress(trapNum) (((trapNum & 0x03FF) << 2) + 0xE00)
#define kMixedModeLibraryName "\pMixedMode"
#define kMM68KContextGlobalName "\pEmulated68KContext"
#define kMMEmulatorReturnGlobalName "\pEmulatorReturn"
#define kMMCallEntryName "\pM68kToPowerPCEntry"
#define kMMReturnEntryName "\pReturnFromM68k"
#define kMMFlushEmulatorCacheName "\pMMFlushEmulatorCacheRange"
#define kSelectorPhase (kCallingConventionWidth + kResultSizeWidth)

enum {
	kMixedModeVersion			= 1
};

extern long gMixedModeVersion;

enum {
	kD0OneByteSelector			= (CallingConventionType)(kD0DispatchedPascalStackBased | (kOneByteCode << (kCallingConventionWidth + kResultSizeWidth))),
	kD0TwoByteSelector			= (CallingConventionType)(kD0DispatchedPascalStackBased | (kTwoByteCode << (kCallingConventionWidth + kResultSizeWidth))),
	kD0FourByteSelector			= (CallingConventionType)(kD0DispatchedPascalStackBased | (kFourByteCode << (kCallingConventionWidth + kResultSizeWidth))),
	kD1OneByteSelector			= (CallingConventionType)(kD1DispatchedPascalStackBased | (kOneByteCode << (kCallingConventionWidth + kResultSizeWidth))),
	kD1TwoByteSelector			= (CallingConventionType)(kD1DispatchedPascalStackBased | (kTwoByteCode << (kCallingConventionWidth + kResultSizeWidth))),
	kD1FourByteSelector			= (CallingConventionType)(kD1DispatchedPascalStackBased | (kFourByteCode << (kCallingConventionWidth + kResultSizeWidth))),
	kStackOneByteSelector		= (CallingConventionType)(kStackDispatchedPascalStackBased | (kOneByteCode << (kCallingConventionWidth + kResultSizeWidth))),
	kStackTwoByteSelector		= (CallingConventionType)(kStackDispatchedPascalStackBased | (kTwoByteCode << (kCallingConventionWidth + kResultSizeWidth))),
	kStackFourByteSelector		= (CallingConventionType)(kStackDispatchedPascalStackBased | (kFourByteCode << (kCallingConventionWidth + kResultSizeWidth)))
};

enum {
	mmNoRDError					= -2527,
	mmNoSelectorError			= -2528,
	kMMErr4						= -2529,
	kMMErr5						= -2530,
	kMMErr6						= -2531,
	kMMErr7						= -2532,
	kMMErr8						= -2533,
	kMMErr9						= -2534,
	kMMErr10					= -2535
};

enum {
	kMMExceptionVector			= 0x0000005c
};

extern OSErr V0InitializeMixedMode(void);
extern OSErr InstallMixedModeMagicTrapHandler(void);
extern OSErr InstallMixedModeExceptionHandler(void);
#if ! SystemEightOrLater
extern OSErr InstallMixedModeGestaltHandler(CFragConnectionID connectionID);
#endif
extern long GetEmulatorRegister(registerSelectorType registerSelector);
extern long SetEmulatorRegister(registerSelectorType registerSelector, long newValue);
extern pascal OSErr SetRoutineRecordProcPtr(short theTrap, long selector, ISAType ISA, ProcPtr newProc, ProcPtr *patchedProc)
 TWOWORDINLINE(0x7003, 0xAA59);
extern pascal OSErr GetRoutineRecordProcPtr(short theTrap, long selector, ISAType ISA, ProcPtr *theProc)
 TWOWORDINLINE(0x7004, 0xAA59);
extern pascal void FlushEmulatorCacheRange(const void *rangeStartAddr, unsigned long rangeLength);
#define kMoveFromECR 0xFE04
#define kMoveToECR 0xFE05
#define kMMSwitcherEntryPtr 0x01
#define kMMSwitcherEntryPtrCall 0x01
#define kMMSwitcherR2 0x02
#define kMMSwitcher68KcontextPtr 0x03
#define kMMSwitcherExitPtr 0x04
#define kMMSwitcherRISCcontextPtr 0x05
#define kATrapHandlerAddr 0x06
#define kATrapOSRtnAddr 0x07
#define kATrapOSNoA0RtnAddr 0x08
#define kMMSwitcherEntryPtrReturn 0x0d
typedef UInt16 EmulatorControlRegisterSelector;


#if !GENERATINGCFM
#pragma parameter __A0 GetSwitcherEntryPtr
#endif
extern Ptr GetSwitcherEntryPtr(void)
 FOURWORDINLINE(0x203C, 0x0000, 0x01, 0xFE04);

#if !GENERATINGCFM
#pragma parameter SetSwitcherEntryPtr(__A0)
#endif
extern void SetSwitcherEntryPtr(long)
 FOURWORDINLINE(0x203c, 0x0000, 0x01, 0xFE05);

#if !GENERATINGCFM
#pragma parameter __A0 GetSwitcherExitPtr
#endif
extern Ptr GetSwitcherExitPtr(void)
 FOURWORDINLINE(0x203C, 0x0000, 0x04, 0xFE04);

#if !GENERATINGCFM
#pragma parameter SetSwitcherExitPtr(__A0)
#endif
extern void SetSwitcherExitPtr(long)
 FOURWORDINLINE(0x203c, 0x0000, 0x04, 0xFE05);

#if !GENERATINGCFM
#pragma parameter __A0 GetSwitcherR2
#endif
extern Ptr GetSwitcherR2(void)
 FOURWORDINLINE(0x203C, 0x0000, 0x02, 0xFE04);

#if !GENERATINGCFM
#pragma parameter SetSwitcherR2(__A0)
#endif
extern void SetSwitcherR2(unsigned long)
 FOURWORDINLINE(0x203C, 0x0000, 0x02, 0xFE05);

#if !GENERATINGCFM
#pragma parameter __A0 Get68kContextPtr
#endif
extern Emulator68KContextBlockPtr Get68kContextPtr(void)
 FOURWORDINLINE(0x203C, 0x0000, 0x03, 0xFE04);

#if !GENERATINGCFM
#pragma parameter __A0 GetRISCContextPtr
#endif
extern RISCContextBlockPtr GetRISCContextPtr(void)
 FOURWORDINLINE(0x203C, 0x0000, 0x05, 0xFE04);

#if !GENERATINGCFM
#pragma parameter __A0 SetMMSwitcherEntryPtrCall(__A0)
#endif
extern RISCContextBlockPtr SetMMSwitcherEntryPtrCall(long)
 FOURWORDINLINE(0x203C, 0x0000, 0x01, 0xFE05);

#if !GENERATINGCFM
#pragma parameter __A0 SetMMSwitcherEntryPtrReturn(__A0)
#endif
extern RISCContextBlockPtr SetMMSwitcherEntryPtrReturn(long)
 FOURWORDINLINE(0x203C, 0x0000, 0x0d, 0xFE05);
extern OSErr SetECR(EmulatorControlRegisterSelector ecr, UInt32 newValue);
extern OSErr GetECR(EmulatorControlRegisterSelector ecr, UInt32 *value);
#ifdef __CFM68K__
#define Style short
typedef short asciichar;
extern UniversalProcPtr CallComponentUPP;
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import off
#endif

#if STRUCTALIGNMENTSUPPORTED
#pragma options align=reset
#endif

#ifdef __cplusplus
}
#endif

#endif /* __MIXEDMODEPRIV__ */
