// ===========================================================================
// UExistingHandleBlock.h
// ===========================================================================


class StExistingHandleBlock {
public:
							StExistingHandleBlock(
								Handle				inExistingHandle);
							~StExistingHandleBlock();

							operator Handle()
							{
								return mHandle;
							}

	void					Dispose();

private:
	Handle					mHandle;
};


template <class T>
class StExistingHandleBlockT {
public:
							StExistingHandleBlockT(
								T 					inExistingHandle);
							~StExistingHandleBlockT();
	
							operator T();
							operator Handle();

	void 					Dispose();
	
private:
	T						mHandle;
};


// ===========================================================================
	template <class T>
StExistingHandleBlockT<T>::StExistingHandleBlockT(
	T						inExistingHandle)
{
	mHandle = inExistingHandle;
}


// ===========================================================================
	template <class T>
StExistingHandleBlockT<T>::~StExistingHandleBlockT()
{
	if (mHandle != nil) {
		DisposeHandle(Handle(mHandle));
	}
}


// ===========================================================================
	template <class T>
StExistingHandleBlockT<T>::operator T()
{ 
	return mHandle; 
}


// ===========================================================================
	template <class T>
StExistingHandleBlockT<T>::operator Handle()
{ 
	return Handle(mHandle); 
}


// ===========================================================================
	template <class T>
	void
StExistingHandleBlockT<T>::Dispose()
{
	if (mHandle != nil) {
		DisposeHandle(Handle(mHandle));
		
		mHandle = nil;
	}
}


// ===========================================================================
//  StExistingSpecialHandleT
// ===========================================================================

template <class T>
class StExistingSpecialHandleT {
public:
							StExistingSpecialHandleT(
								T 					inExistingSpecialHandle,
								pascal void 		(*inHandleDisposerFuncPtr)(T));
								
							~StExistingSpecialHandleT();
	
							operator T();
							operator Handle();

	void 					Dispose();
	
private:
	T						mHandle;
	pascal void 			(*mHandleDisposerFuncPtr)(T);
};


// ===========================================================================
	template <class T>
StExistingSpecialHandleT<T>::StExistingSpecialHandleT(
	T						inExistingSpecialHandle,
	pascal void 			(*inHandleDisposerFuncPtr)(T))
{
	mHandle = inExistingSpecialHandle;
	mHandleDisposerFuncPtr = inHandleDisposerFuncPtr;
}


// ===========================================================================
	template <class T>
StExistingSpecialHandleT<T>::~StExistingSpecialHandleT()
{
	if (mHandle != nil) {
		(*mHandleDisposerFuncPtr)(mHandle);
	}
}


// ===========================================================================
	template <class T>
StExistingSpecialHandleT<T>::operator T()
{ 
	return mHandle; 
}


// ===========================================================================
	template <class T>
StExistingSpecialHandleT<T>::operator Handle()
{ 
	return Handle(mHandle); 
}


// ===========================================================================
	template <class T>
	void
StExistingSpecialHandleT<T>::Dispose()
{
	if (mHandle != nil) {
		(*mHandleDisposerFuncPtr)(mHandle);
		
		mHandle = nil;
	}
}
