// ===========================================================================
//	CAboutBox.cp
// ===========================================================================


#include "CAboutBox.h"


const ResIDT	WIND_AboutBox		= 4000;


CAboutBox* CAboutBox::sAboutBox = nil;


// ===========================================================================
	void
CAboutBox::ShowSplashScreen(
	LCommander	*inSuperCommander)
{
	if (sAboutBox == nil) {
		sAboutBox = CreateAboutBox(WIND_AboutBox, inSuperCommander);
		Assert_(sAboutBox);
		
			// force the about box to update.  we have to do this because we don't
			// give the system any time to send us an update event.
		sAboutBox->Draw(nil);
	}
}


// ===========================================================================
	void
CAboutBox::HideSplashScreen()
{
	delete sAboutBox;
	sAboutBox = nil;
}


// ===========================================================================
	void
CAboutBox::ShowAboutBox(
	LCommander	*inSuperCommander)
{
	if (sAboutBox == nil) {
		sAboutBox = CreateAboutBox(WIND_AboutBox, inSuperCommander);
		Assert_(sAboutBox);
	}
}


// ===========================================================================
	void
CAboutBox::HideAboutBox()
{
	delete sAboutBox;
	sAboutBox = nil;
}


// ===========================================================================
//		 CreateAboutBox
// ===========================================================================
//	Return a newly created Window object initialized from a PPob resource

	CAboutBox*
CAboutBox::CreateAboutBox(
	ResIDT		inWindowID,
	LCommander	*inSuperCommander)
{
	SetDefaultCommander(inSuperCommander);

	CAboutBox* theAboutBox =
			(CAboutBox*) UReanimator::ReadObjects('PPob', inWindowID);

	theAboutBox->FinishCreate();

		// we always want to show the about box as soon as it's created 
	theAboutBox->Show();
	
	return theAboutBox;
}


// ===========================================================================
//		 CreateSettingsWindowStream
// ===========================================================================
//	Return a newly created Window object initialized with data from a Stream

	CAboutBox*
CAboutBox::CreateAboutBoxStream(
	LStream	*inStream)
{
	return (new CAboutBox(inStream));
}


// ===========================================================================
//		 CAboutBox(LStream*)
// ===========================================================================
//	Construct Window from the data in a stream

CAboutBox::CAboutBox(
	LStream	*inStream)
	: LDialogBox(inStream)
{
}


// ===========================================================================
CAboutBox::~CAboutBox()
{
		// we're going away, so remove the pointer to this about box from the
		// static member
	sAboutBox = nil;
}


// ===========================================================================
	void
CAboutBox::ClickInContent(
	const EventRecord	& /* inMacEvent */)
{
		// close the about box when the user clicks inside it
	DoClose();
}


// ===========================================================================
	Boolean
CAboutBox::HandleKeyPress(
	const EventRecord	&inKeyEvent)
{
	Boolean		keyHandled = false;
	LControl	*keyButton = nil;
	
		// return, enter, escape and command-period will close the about box
	switch (inKeyEvent.message & charCodeMask) {
	
		case char_Enter:
		case char_Return:
		case char_Escape:
			keyHandled = true;
			DoClose();
			break;

		default:
			if (UKeyFilters::IsCmdPeriod(inKeyEvent)) {
				keyHandled = true;
				DoClose();
			} else {
				keyHandled = LWindow::HandleKeyPress(inKeyEvent);
			}
			break;
	}
			
	return keyHandled;
}


// ===========================================================================
	void
CAboutBox::FindCommandStatus(
	CommandT	inCommand,
	Boolean		&outEnabled,
	Boolean		&outUsesMark,
	Char16		&outMark,
	Str255		outName)
{
	#pragma unused(inCommand, outMark, outName)
	
		// when the about box is open, all of the menu items will be disabled, 
		// except for About...  if we disable About..., the whole Apple menu is
		// disabled, which we don't want.  it's okay to enable About... while
		// the about box is showing because selecting it again will have no effect.
	outUsesMark = false;
	outEnabled = false;

	if (inCommand == cmd_About) {
		outEnabled = true;
	}
}