// ===========================================================================
//	CCLuTPopupMenu.cp
// ===========================================================================


#include <Processes.h>
#include <Resources.h>
#include <Files.h>
#include <string.h>
#include <limits.h>
#include <stdlib.h>

#include "CCLuTPopupMenu.h"
#include "CPICTPopupMenu.h"
#include "StringConversion.h"
#include "Search.h"
#include "S3Utilities.h"
#include "S3PICTList.h"


const short kChooseCLuTItem = 2;


MenuHandle CCLuTPopupMenu::sMenuH = nil;
ResIDT CCLuTPopupMenu::sMenuID = 0;
LList CCLuTPopupMenu::sPopupMenuList;
LList CCLuTPopupMenu::sDataList(sizeof(FSSpec));
Boolean CCLuTPopupMenu::sDoUpdate = true;
Boolean CCLuTPopupMenu::sAddedDivider = false;


// ===========================================================================
	void
CCLuTPopupMenu::InitializeMenu()
{
		// find out where the app was launched from so that we can look for 
		// rsrc files in the same folder
	FSSpec appFileSpec;
	S3GetAppFileSpec(&appFileSpec);
	
		// set up two CInfoPBRec's to pass to IndexedSearch()
	CSParam paramBlock;
	CInfoPBRec searchInfo1, searchInfo2;

		// we want files only, so clear bit 4 (1 == directory, 0 == file) 
		// and set bit 4 in the mask
	searchInfo1.hFileInfo.ioFlAttrib = 0x00;
	searchInfo2.hFileInfo.ioFlAttrib = 0x10;

		// we don't care what names the files have
	searchInfo1.hFileInfo.ioNamePtr = nil;
	searchInfo2.hFileInfo.ioNamePtr = nil;

		// we want to search on the fdType, but none of the other fields.  for
		// some reason, setting all the searchInfo2 fields to 0 except for 
		// fdType (which is the one we want) seem to give the right result.
		// the searchInfo2 fields are supposed to just be a mask.
	searchInfo1.hFileInfo.ioFlFndrInfo.fdType = kCLuT_FileType;
	searchInfo2.hFileInfo.ioFlFndrInfo.fdType = kCLuT_FileType;
	searchInfo2.hFileInfo.ioFlFndrInfo.fdCreator = 0;
	searchInfo2.hFileInfo.ioFlFndrInfo.fdFlags = 0;
	searchInfo2.hFileInfo.ioFlFndrInfo.fdLocation.h = 0;
	searchInfo2.hFileInfo.ioFlFndrInfo.fdLocation.v = 0;
	searchInfo2.hFileInfo.ioFlFndrInfo.fdFldr = 0;

		// we want to find files in or below the app's folder only
	paramBlock.ioVRefNum					= appFileSpec.vRefNum;

		// we don't have a completion routine and don't care about the name
		// of the current file
	paramBlock.ioCompletion					= nil;
	paramBlock.ioNamePtr					= nil;
	
		// create an array of FSSpecs in which to store the results of the
		// indexed search
	FSSpec* CLuTFileSpecs = new FSSpec[kMaxCLuTCount];
	
	if (CLuTFileSpecs == nil) {
		return;
	}
	
		// initialize the array of FSSpec's to 0.  this is important because
		// LDynamicArray::FetchIndexOf uses BlocksAreEqual to find the index
		// of a piece of data.  as there's a string in FSSpec, two FSSpec's
		// might have the same data but not be equal because they have different
		// garbage following the end of the string.
	memset(CLuTFileSpecs, unsigned char(0), sizeof(FSSpec) * kMaxCLuTCount);
	
	paramBlock.ioMatchPtr					= CLuTFileSpecs;

		// make at most kMaxCLuTCount matches before returning
	paramBlock.ioReqMatchCount					= kMaxCLuTCount;

		// search according to the attributes in the 
		// ioFlFndrInfo and ioFlAttrib fields
	paramBlock.ioSearchBits					= fsSBFlFndrInfo + fsSBFlAttrib;
	
	paramBlock.ioSearchInfo1					= &searchInfo1;
	paramBlock.ioSearchInfo2					= &searchInfo2;
	paramBlock.ioSearchTime					= 0;
	paramBlock.ioCatPosition.initialize = 0;
	paramBlock.ioOptBuffer 				= nil;
	paramBlock.ioOptBufSize				 = 0;

	OSErr err = IndexedSearch(&paramBlock, appFileSpec.parID);
	
	if ((err == noErr || err == eofErr) && (paramBlock.ioActMatchCount > 0)) {
			// first sort the file specs, since they come back with files in a
			// subfolder inserted in the list where the subfolder is listed in
			// the parent folder.  so if folder f1 contains A B f2 D E, and subfolder
			// f2 contains X Y Z, the list will be A B X Y Z D E, which we want to fix.
		qsort(CLuTFileSpecs, paramBlock.ioActMatchCount, sizeof(FSSpec), S3FSSpecComparator);
		
			// let the popup know we're going to be adding a bunch of items in one
			// big group
		CCLuTPopupMenu::BeginUpdate();
		
			// save the currently opened resource file so we can reopen it 
			// when we're done
		short 					savedResFile = ::CurResFile();
		short 					numCLuTs = 0;
		short 					currentRefNum = 0;
		
		for (short i = 0; i < paramBlock.ioActMatchCount; i++) {
			currentRefNum = ::FSpOpenResFile(&CLuTFileSpecs[i], fsRdPerm);
			
				// ignore any errors we might encounter
			if (::ResError() != noErr) continue;
			
			numCLuTs = ::Count1Resources(kCLuT_ResourceType);
			
			if (::ResError() != noErr || numCLuTs == 0) {
				continue;
			}
			
			Handle 					hCLuT = nil;
			LStr255 				menuItemName, resourceName;
			short 					resourceID;
			ResType 				resourceType;
			
				// get a handle to each CLuT in this file, one at a time
			for (short iCLuT = 1; iCLuT <= numCLuTs; iCLuT++) {	
				hCLuT = ::Get1IndResource(kCLuT_ResourceType, iCLuT);
				
				if (::ResError() != noErr) goto cleanupResourceHandle; 

					// get info about the current CLuT resource.  we need
					// this so we can create a name for it in the menu.
				::GetResInfo(hCLuT, &resourceID, &resourceType, resourceName);

					// create a name for this CLuT in the popup menu.  the
					// name consists of the file name the resource is from,
					// a colon, and the resource name if it is non-blank, 
					// otherwise the resource's ID.
				if (resourceName.Length() > 0) {
					menuItemName = CLuTFileSpecs[i].name;
					menuItemName += "\p : ";
					menuItemName += resourceName;
//					((menuItemName = CLuTFileSpecs[i].name) += "\p : ") += resourceName;
				} else {
					menuItemName = CLuTFileSpecs[i].name;
					menuItemName += "\p : ";
					
						// cast the resource ID to a Int32 so LStr255 knows 
						// how to convert it to a string
					menuItemName += LStr255(Int32(resourceID));
//					menuItemName = CLuTFileSpecs[i].name + "\p : " + resourceID;
				}
				
				CCLuTPopupMenu::AppendSharedItem(
						menuItemName, &(CLuTFileSpecs[i]));

cleanupResourceHandle:				
				if (hCLuT != nil) {
					::ReleaseResource(hCLuT);
				}
			} // endfor
			
				// we're done with this resource file
			::CloseResFile(currentRefNum);
		} // endfor
		
			// go back to the previous resource file
		::UseResFile(savedResFile);
		
		CCLuTPopupMenu::EndUpdate();
	}
	
	delete[] CLuTFileSpecs;
}


// ===========================================================================
	void
CCLuTPopupMenu::ReinitializeMenu()
{
	// first get rid of old data values
	// then call InitializeMenu()
	InitializeMenu();
}


// ===========================================================================
	void
CCLuTPopupMenu::UpdatePopupMenus()
{
	if (!sDoUpdate) {
			// we're in the middle of a BeginUpdate/EndUpdate pair, so we don't
			// do the update until EndUpdate is called
		return;
	}
	
	LListIterator iterator(sPopupMenuList, iterate_FromStart);
	CCLuTPopupMenu* thePopup = nil;

		// update the control values of each open popup
	while (iterator.Next(&thePopup)) {
		Assert_(thePopup);
		
		thePopup->UpdateValues();
	}
}


// ===========================================================================
	void
CCLuTPopupMenu::BeginUpdate()
{
		// we set this flag to false so that UpdatePopupMenus() doesn't update
		// every popup after each call to AppendSharedItem, InsertSharedItem, etc.
		// when EndUpdate is called, this flag will be reset.
	sDoUpdate = false;
}


// ===========================================================================
	void
CCLuTPopupMenu::EndUpdate()
{
		// turn updating back on
	sDoUpdate = true;
	
		// make sure all the popups get updated correctly, now that the caller
		// is done changing the shared menu.
	UpdatePopupMenus();
}


// ===========================================================================
// only one item can be appended at a time
//
	void
CCLuTPopupMenu::AppendSharedItem(const Str255 inItemString, void* inData, Boolean inIgnoreMetaCharacters)
{
	if (inIgnoreMetaCharacters) {
			// first add the item with a bogus string, then set the item's text
			// to the actual string.  we do this so that we can use metacharacters
			// in the item's name.
		::AppendMenu(sMenuH, "\pTemporary Item String");
		::SetMenuItemText(sMenuH, ::CountMItems(sMenuH), inItemString);
	} else {
			// the caller wants the metacharacters in inItemString to work, so
			// append the item the normal way
		::AppendMenu(sMenuH, inItemString);
	}
	
		// append the data for this item 
	sDataList.InsertItemsAt(1, arrayIndex_Last, inData);
	
	UpdatePopupMenus();
}


// ===========================================================================
// only one item can be inserted at a time
//
	void
CCLuTPopupMenu::InsertSharedItem(const Str255 inItemString, Int16 inAfterItem, void* inData, Boolean inIgnoreMetaCharacters)
{
	if (inIgnoreMetaCharacters) {
			// first add the item with a bogus string, then set the item's text
			// to the actual string.  we do this so that we can use metacharacters
			// in the item's name.
		::InsertMenuItem(sMenuH, "\pTemporary Item String", inAfterItem);
		::SetMenuItemText(sMenuH, inAfterItem + 1, inItemString);
	} else {
			// the caller wants the metacharacters in inItemString to work, so
			// insert the item the normal way
		::InsertMenuItem(sMenuH, inItemString, inAfterItem);
	}
	
		// insert the data for this item.  inAfterItem is the index of the item
		// after which the new item is inserted, so add 1 to it get the LList index.
	sDataList.InsertItemsAt(1, inAfterItem + 1, inData);
	
	UpdatePopupMenus();
}


// ===========================================================================
	void
CCLuTPopupMenu::AppendPossiblyNewItem(const Str255 inItemString, void* inData)
{
	BeginUpdate();
	
	if (GetSharedItemFromData(inData) > 0) {
			// this FSSpec is already in the CLuT list, so don't do anything different
		return;
	}
	
		// append a divider to the end ofthe list (if we haven't already)
	AppendDivider();
	
		// append the texture 
	AppendSharedItem(inItemString, inData);
	
		// force all the existing CLuT popups to update
	EndUpdate();
}


//// ===========================================================================
//	void
//CCLuTPopupMenu::AppendSharedResourceItems(ResType inResType, void* inData)
//{
//	::AppendResMenu(GetMacMenuH(), inResType);
//	
//	UpdatePopupMenus();
//}
//
//
//// ===========================================================================
//	void
//CCLuTPopupMenu::InsertSharedResourceItems(ResType inResType, Int16 inAfterItem)
//{
//	::InsertResMenu(GetMacMenuH(), inResType, inAfterItem);
//	
//	UpdatePopupMenus();
//}


// ===========================================================================
	void
CCLuTPopupMenu::DeleteSharedItem(Int16 inItemNumber)
{
	::DeleteMenuItem(sMenuH, inItemNumber);
	
		// remove the item's corresponding data 
	sDataList.RemoveItemsAt(1, inItemNumber);
	
	UpdatePopupMenus();
}


// ===========================================================================
	Int16
CCLuTPopupMenu::GetSharedItemFromString(const Str255 inString)
{
		// Search for inString in the items of this Menu
	Str255 itemString;
	short numItems = ::CountMItems(sMenuH);
	
	for (short theIndex = 1; theIndex <= numItems; theIndex++) {
		::GetMenuItemText(sMenuH, theIndex, itemString);
		
		if (::EqualString(inString, itemString, true, false)) {
			return theIndex;
		}
	}
	
		// the string wasn't found in this menu
	return 0;
}


// ===========================================================================
	Int16
CCLuTPopupMenu::GetSharedItemFromString(const char* inString)
{
	Str255 PString;
	S3CtoPString(inString, PString);
	
			// Search for inString in the items of this Menu
	Str255 itemString;
	short numItems = ::CountMItems(sMenuH);
	
	for (short theIndex = 1; theIndex <= numItems; theIndex++) {
		::GetMenuItemText(sMenuH, theIndex, itemString);
		
		if (::EqualString(PString, itemString, true, false)) {
			return theIndex;
		}
	}
	
		// the string wasn't found in this menu
	return 0;
}


// ===========================================================================
	Boolean
CCLuTPopupMenu::GetSharedDataFromString(const Str255 inString, void* outData)
{
	Int16 index = GetSharedItemFromString(inString);
	
	return GetSharedDataFromItem(index, outData);
}


// ===========================================================================
	Boolean
CCLuTPopupMenu::GetSharedDataFromString(const char* inString, void* outData)
{
	Int16 index = GetSharedItemFromString(inString);
	
	return GetSharedDataFromItem(index, outData);
}


// ===========================================================================
	Boolean
CCLuTPopupMenu::GetSharedDataFromItem(Int16 inIndex, void* outData)
{
		// if the index is out of bounds, outData is unchanged and FetchItemAt
		// returns false
	return sDataList.FetchItemAt(inIndex, outData);
}


// ===========================================================================
	Int16
CCLuTPopupMenu::GetSharedItemFromData(const void* inData)
{
		// if the index is out of bounds, data will remain nil
	Int32 index = sDataList.FetchIndexOf(inData);
	
	if (index < 1) {
			// the data wasn't found in the list
		return 0;
	}
	
	return (Int16) index;
}


// ===========================================================================
//  CCLuTPopupMenu											   CCLuTPopupMenu 
// ===========================================================================
	CCLuTPopupMenu*
CCLuTPopupMenu::CreateCLuTPopupMenuStream(LStream	*inStream)
{
	return (new CCLuTPopupMenu(inStream));
}


// ===========================================================================
//		 CCLuTPopupMenu
// ===========================================================================
//	Construct from input parameters

CCLuTPopupMenu::CCLuTPopupMenu(
	const SPaneInfo	&inPaneInfo,
	MessageT		inValueMessage,
	Int16			inTitleOptions,
	ResIDT			inMENUid,
	Int16			inTitleWidth,
	Int16			inPopupVariation,
	ResIDT			inTextTraitsID,
	Str255			inTitle,
	OSType			inResTypeMENU,
	Int16			inInitialMenuItem)
		:	CSmartStdPopupMenu(inPaneInfo, inValueMessage, inTitleOptions,
				inMENUid, inTitleWidth, inPopupVariation, inTextTraitsID,
				inTitle, inResTypeMENU, inInitialMenuItem)
{
	InitCLuTPopupMenu();
}


// ===========================================================================
//		 CCLuTPopupMenu
// ===========================================================================
//	Construct from input parameters and an existing ControlHandle

CCLuTPopupMenu::CCLuTPopupMenu(
	const SPaneInfo	&inPaneInfo,
	MessageT		inValueMessage,
	Int32			inMaxValue,
	ResIDT			inTextTraitsID,
	ControlHandle	inMacControlH)
		: CSmartStdPopupMenu(inPaneInfo, inValueMessage, inMaxValue,
						inTextTraitsID, inMacControlH)
{
	InitCLuTPopupMenu();
}


// ===========================================================================
//		 CCLuTPopupMenu(LStream*)
// ===========================================================================
//	Construct from data in a Stream

CCLuTPopupMenu::CCLuTPopupMenu(
	LStream	*inStream)
		:	CSmartStdPopupMenu(inStream)
{	
	InitCLuTPopupMenu();
}


// ===========================================================================
//		 ~CCLuTPopupMenu
// ===========================================================================
CCLuTPopupMenu::~CCLuTPopupMenu()
{	
		// remove us from the list of popup menus
	sPopupMenuList.Remove(&this);
	
		// get a handle to the popup's control record
	ControlHandle controlH = GetMacControl();
	
		// get a handle to the popup's private data record in the control record
	PopupPrivateDataHandle	dataH =
		(PopupPrivateDataHandle) (**controlH).contrlData;

		// we duplicate the menu and stick the copy in the control's private
		// data so that when ~LStdControl disposes it, the copy will be destroyed
		// and not the menu all the remaining versions of this popup are 
		// pointing to.
	::HandToHand((Handle*) &(**dataH).mHandle);
}

// ===========================================================================
//		 InitCLuTPopupMenu
// ===========================================================================
//	Private initializer

	void
CCLuTPopupMenu::InitCLuTPopupMenu()
{	
		// add us to the list of existing popup menus.  we have to do this 
		// before initialzing the menu for the first time, because anything
		// that gets added to the menu won't show up in the very first popup
		// unless there's a pointer to it in the sPopupMenuList.
	sPopupMenuList.InsertItemsAt(1, arrayIndex_Last, &this);
	
	if (sMenuH == nil) {
			// this is the first popup instance created, so we have to copy its
			// menu and dispose of the original.  if we don't, the menu shared
			// by all the popups in this class will be disposed when one of them 
			// is destroyed, so the remaining ones won't have any menu to use,
			// causing all sorts of errors.
		PopupPrivateDataHandle	dataH =
			(PopupPrivateDataHandle) (**GetMacControl()).contrlData;
	
		MenuHandle menuH = (**dataH).mHandle;
		MenuHandle menuCopyH = menuH;
		::HandToHand((Handle*) &menuCopyH);
		
			// dispose of the existing menu
		::ReleaseResource((Handle) menuH);

			// turn off the purgeable and resource bits in the menu handle we
			// just copied.  this may not be necessary, but it doesn't hurt anyting.
		::HNoPurge((Handle) menuCopyH);
		::HClrRBit((Handle) menuCopyH);

			// insert the copied menu as a popup
		::InsertMenu(menuCopyH, -1);

			// all the instances created after this will use this menu and ID
		sMenuH = menuCopyH;
		sMenuID = (**sMenuH).menuID;
		
			// make the menu handle of this popup point to the copied menu
		(**dataH).mHandle = sMenuH;

		Int32 itemCount = GetMaxValue();
		Int32 dataCount = sDataList.GetCount();
		
			// either of these two branches can be true only if this is the
			// first popup menu to be created.  after that, the insertion and
			// deletion methods should keep the data list and the menu in sync.
		if (itemCount > dataCount) {
				// the menu has more items than we have data values for, so append
				// an empty FSSpec to the end of sDataList, one for each item that 
				// has no data
			FSSpec emptyFSSpec = { 0, 0, { 0 } };
			sDataList.InsertItemsAt(itemCount - dataCount,
														  arrayIndex_Last, &emptyFSSpec);
		} else if (itemCount < dataCount) {
				// the menu has fewer items than we have data values for, so remove
				// the extra data values from the end of sDataList
			sDataList.RemoveItemsAt(dataCount - itemCount, itemCount + 1);
		}
		
			// we also have to initialize the menu data to hold the CLuT file specs
		InitializeMenu();
	}
	
	 // save the string of the item currently selected in the popup
	GetStringFromItem(GetValue(), mValueString);
	
		// don't broadcast a string as our message value
	SetBroadcastString(false);
}


// ===========================================================================
	void
CCLuTPopupMenu::SetValue(Int32 inValue)
{
	inherited::SetValue(inValue);
	
		// save the string of the item we just set
	GetStringFromItem(GetValue(), mValueString);
}


// ===========================================================================
	void
CCLuTPopupMenu::SetValueByData(const void* inData)
{	
	Int16 index = GetSharedItemFromData(inData);
	
	if (index == 0) {
			// this string wasn't found in the menu, so default to the first item
		index = 1;
	}
	
	SetValue(index);
}


// ===========================================================================
	Boolean
CCLuTPopupMenu::GetData(void* outData) const
{
		// get the data for the currently selected item and return it in outData
	return GetSharedDataFromItem(GetValue(), outData);
}


// ===========================================================================
	void
CCLuTPopupMenu::AppendItem(const Str255 inItemString)
{
	AppendSharedItem(inItemString);
}


// ===========================================================================
	void
CCLuTPopupMenu::InsertItem(const Str255 inItemString, Int16 inAfterItem)
{
	InsertSharedItem(inItemString, inAfterItem);
}


//// ===========================================================================
//	void
//CCLuTPopupMenu::AppendResourceItems(ResType inResType)
//{
//	AppendSharedResourceItems(inResType);
//}
//
//
//// ===========================================================================
//	void
//CCLuTPopupMenu::InsertResourceItems(ResType inResType, Int16 inAfterItem)
//{
//	InsertSharedResourceItems(inResType, inAfterItem);
//}


// ===========================================================================
	void
CCLuTPopupMenu::DeleteItem(Int16 inItemNumber)
{
	DeleteSharedItem(inItemNumber);
}


// ===========================================================================
	void
CCLuTPopupMenu::UpdateValues()
{
		// determine the maximum value by counting the number of items in 
		// the menu resource
	SetMaxValue(::CountMItems(GetMacMenuH()));
	
	if (GetMaxValue() > 0) {
			// there's at least one item in the menu, so make sure the min
			// value is 1, since we might have just added the first item in the
			// menu, and the min value might still be 0
		SetMinValue(1);
		
			// if items in the menu were changed, the text of the item with 
			// the same number as the value of the popup might be different 
			// than what it was before, because the order of the items has
			// shifted, or the previously selected item was deleted.  although
			// the popup's value is still the same, to the user it won't look 
			// like the same item is selected since a different string appears
			// next to the title.  so reset the value according to the string
			// of the item that was previously selected.  if the item has been
			// deleted (and thus its string can't be found in the menu), 
			// SetValueByString will default to item 1.
		SetValueByString(mValueString);
	} else {
			// there are no items in the menu, so set the other values to
			// reflect that
		SetMinValue(0);
		SetValue(0);
	}
}


// ===========================================================================
	void
CCLuTPopupMenu::BroadcastValueMessage()
{
	if (mValueMessage != cmd_Nothing) {
		if (GetValue() == kChooseCLuTItem) {
				// before we broadcast a value, we have to let the user pick a CLuT
				// in a standard get file, and then broadcast that file spec
			ChooseCLuT();
		}
		
		if (mBroadcastString) {
			Str255 theString;
			GetString(theString);
			
			BroadcastMessage(mValueMessage, (void*) theString);
		} else {
// should have a method to switch between several broadcasting types:
// string, pointer, value, data
//			Int32	value = GetValue();
//
//			BroadcastMessage(mValueMessage, (void*) &value);

				// broadcast a pointer to this instance
			BroadcastMessage(mValueMessage, (void*) this);
		}
	}
}


// ===========================================================================
	void
CCLuTPopupMenu::ChooseCLuT()
{
/*	
		// we have to initialize the FSSpec to all 0s so that the LList will be
		// able to find it, since all the FSSpecs in the list were also
		// initialized to 0
	FSSpec newCLuT = { 0, 0, { 0 } };
	
	if (S3GetCLuTFileSpec(&newCLuT)) {
			// try to append the texture the user just chose
		AppendPossiblyNewItem(newCLuT.name, &newCLuT);
	}

		// select the texture in the list.  if the user canceled the file dialog,
		// this will select None, because it's too much of a pain to remember
		// their previous selection.
	SetValueByData(&newCLuT);
*/
}


// ===========================================================================
	void
CCLuTPopupMenu::AppendDivider()
{
	if (!sAddedDivider) {
			// we haven't added a divider to the end of the CLuT list yet,
			// so do that first
		FSSpec emptyFSSpec = { 0, 0, { 0 } };
		AppendSharedItem("\p-", &emptyFSSpec, false);
		sAddedDivider = true;
	}
}


	// external C interface to this class for getting a filespec stored in
	// the popup menu from the file's name
extern "C" {

	
//// ===========================================================================
//	Boolean
//S3CLuTList_GetFileSpec(
//	S3CLuTList		inCLuTList,
//	char*					inFileName,
//	FSSpec* 			outFSSpec)
//{
//	Assert_(inCLuTList);
//	Assert_(inFileName);
//	Assert_(outFSSpec);
//	
//	CCLuTPopupMenu*	popup = (CCLuTPopupMenu*) inCLuTList;
//	
//	return popup->GetSharedDataFromString(inFileName, outFSSpec);
//}


//// ===========================================================================
//	int
//S3FSSpecComparator(
//	const void*		inFSSpec1,
//	const void*		inFSSpec2)
//{
//		// we sort the FSSpecs based on the relative sorting of the filenames 
//		// they contain, after first casting the void pointers to FSSpec*.
//		// passing nil causes CompareString to use the string comparison info 
//		// for the current script.
//	return ::CompareString(((FSSpec*) inFSSpec1)->name, ((FSSpec*) inFSSpec2)->name, nil);
//}

	
}
