// ===========================================================================
// CDottedLine.cp
// ===========================================================================


#include <QuickDraw.h>
#include <UDrawingState.h>

#include "CDottedLine.h"


// ---------------------------------------------------------------------------
//		 CreatePaneStream [static]
// ---------------------------------------------------------------------------
//	Return a new Pane object initialized using data from a Stream

	CDottedLine*
CDottedLine::CreateDottedLineStream(
	LStream	*inStream)
{
	return (new CDottedLine(inStream));
}


// ---------------------------------------------------------------------------
//		 CDottedLine()
// ---------------------------------------------------------------------------
//	Default Constructor

CDottedLine::CDottedLine()
		: LPane()
{
}


// ---------------------------------------------------------------------------
//		 CDottedLine(const LPane&)
// ---------------------------------------------------------------------------
//	Copy Constructor

CDottedLine::CDottedLine(
	const LPane	&inOriginal)
		: LPane(inOriginal)
{
}


// ---------------------------------------------------------------------------
//		 CDottedLine(SPaneInfo&)
// ---------------------------------------------------------------------------
//	Construct Pane from data in a struct

CDottedLine::CDottedLine(
	const SPaneInfo	&inPaneInfo)
		: LPane(inPaneInfo)
{
}


// ---------------------------------------------------------------------------
//		 CDottedLine(LStream*)
// ---------------------------------------------------------------------------
//	Construct Pane from data in a Stream

CDottedLine::CDottedLine(
	LStream	*inStream)
		: LPane(inStream)
{
}


// ===========================================================================
//	Derived classes must override this function in order to draw. Normally,
//	you will call CalcLocalFrameRect to get a QuickDraw rectangle defining
//	the size and location of the Pane. Perform all drawing operations
//	relative to the frame rectangle. For example:
//
//		::PenNormal();
//		ApplyForeAndBackColors();
//		Rect	frame;
//		CalcLocalFrameRect(frame);
//		::FrameRect(&frame);
//		::MoveTo(frame.left, frame.top);
//		::LineTo(frame.right, frame.bottom);
//
//			This draws a box around the Pane and a diagonal line from
//			the top left to the bottom right corner using the default
//			foreground and background colors for the Pane.
//
//	Clipping:
//	On entry, the clipping region is the revealed area of the Pane's
//	SuperView. Therefore, it is possible for a Pane to draw outside of
//	its Frame. You will not normally do this.
//
//	Usage Note: The port, coordinate system, and clipping region are
//		set on entry. They must be the same upon exit. You are responsible
//		for setting the Pen state and text characteristics to the proper
//		values for your Pane.

	void
CDottedLine::DrawSelf()
{
		// save the current GrafPort info
	GrafPtr currentPort;
	::GetPort(&currentPort);
	StColorPortState savedPortState(currentPort);
	
		// set the pen to 1 X 1, copy mode, with a grey (dotted) pattern
	::PenNormal();
	::PenPat(&qd.gray);
	
		// get the width and height in local coords for this pane
	Rect frame;
	CalcLocalFrameRect(frame);
	Int16 paneWidth = frame.right - frame.left;
	Int16 paneHeight = frame.bottom - frame.top;

		// set the start and end points of the line, which is always drawn
		// along the pane's longest axis.  if the pane is square, draw the
		// line horizontally.
	Point lineStart, lineEnd;
	
	if (paneHeight > paneWidth) {
			// the frame is taller than it is wide
		lineStart.h = frame.left + paneWidth / 2;
		lineStart.v = frame.top;
		
		lineEnd.h = frame.left + paneWidth / 2;
		lineEnd.v = frame.bottom;
	} else {
			// the frame is wider than it is tall
		lineStart.h = frame.left;
		lineStart.v = frame.top + paneHeight / 2;
		
		lineEnd.h = frame.right;
		lineEnd.v = frame.top + paneHeight / 2;
	}
	
	::MoveTo(lineStart.h, lineStart.v);
	::LineTo(lineEnd.h, lineEnd.v);
}

