// ===========================================================================
// CEnablerView.cp
// ===========================================================================


#include "CEnablerView.h"


// ===========================================================================
	CEnablerView*
CEnablerView::CreateEnablerViewStream(
	LStream*				inStream)
{
	return new CEnablerView(inStream);
}


// ===========================================================================
CEnablerView::CEnablerView()
	: LView()
{
	mEnablerFunction = nil;
}


// ===========================================================================
CEnablerView::CEnablerView(
	const CEnablerView 		&inOriginal)
		: LView(inOriginal)
{
	mEnablerFunction = inOriginal.mEnablerFunction;
}


// ===========================================================================
CEnablerView::CEnablerView(
	const SPaneInfo&		inPaneInfo,
	const SViewInfo&		inViewInfo,
	EnablerFunctionPtr		inEnablerFunction)
		: LView(inPaneInfo, inViewInfo)
{
	Assert_(inEnablerFunction);
	
	mEnablerFunction = inEnablerFunction;
}


// ===========================================================================
CEnablerView::CEnablerView(
	LStream*				inStream)
		: LView(inStream)
{
	mEnablerFunction = nil;
}


// ===========================================================================
CEnablerView::~CEnablerView()
{
}


// ===========================================================================
	void
CEnablerView::SetEnablerFunction(
	EnablerFunctionPtr		inEnablerFunction)
{
	Assert_(inEnablerFunction);
	
	mEnablerFunction = inEnablerFunction;
}


// ===========================================================================
	void
CEnablerView::ListenToMessage(
	MessageT				inMessage,
	void*					ioParam)
{
	Assert_(mEnablerFunction);
	
	switch ((*mEnablerFunction)(inMessage, ioParam)) {
		case eEnablerResult_Enable:
			Enable();
			break;
		
		case eEnablerResult_Disable:
			Disable();
			break;
		
		case eEnablerResult_IgnoreMessage:
			break;
	} // endswitch
}


// ===========================================================================
	Boolean
CEnablerView::ObeyCommand(
	CommandT				inCommand,
	void*					ioParam)
{
	Boolean	cmdHandled = true;
	
	Assert_(mEnablerFunction);
	
	switch ((*mEnablerFunction)(inCommand, ioParam)) {
		case eEnablerResult_Enable: 
			Enable();
			break;
		
		case eEnablerResult_Disable:
			Disable();
			break;
		
		case eEnablerResult_IgnoreMessage:
		default:
			cmdHandled = mSuperCommander->ObeyCommand(inCommand, ioParam);
			break;
	}
	
	return cmdHandled;
}
