// ===========================================================================
//	CEnginePopupMenu.cp
// ===========================================================================
//
// This derivative of a standard popup menu will broadcast the item text
// of the selected item, rather than the item's number.  The current item
// can also be set by giving the menu a string, which it converts to an 
// item number.


#ifdef PowerPlant_PCH
#include PowerPlant_PCH
#endif


#include <PP_Messages.h>

#include "CEnginePopupMenu.h"
#include "S3Utilities.h"
#include "S3TestInfo.h"
#include "StringConversion.h"


MenuHandle CEnginePopupMenu::sMenuH = nil;
ResIDT CEnginePopupMenu::sMenuID = 0;
LList CEnginePopupMenu::sPopupMenuList;
//LList CEnginePopupMenu::sDataList(sizeof(FSSpec));
Boolean CEnginePopupMenu::sDoUpdate = true;


// ===========================================================================
	void
CEnginePopupMenu::InitializeMenu()
{
		// this is a magic call to enable the Gotham card so that it will 
		// act as a TinselTown drawing engine.  it should then show up in
		// our list of engines.
	QAEngineEnable(0, -1);
	
	if (::CountMItems(sMenuH) == 0) {
			// the engine popup is empty, so get the names of all available engines
			// and add them to the menu.  this should only be done once.
		TQAEngine* engine;
		char currentEngineName[kMaxEngineNameLength];
		long currentEngineNameLength = 0;
		short numItems = 0;
	
			// loop through every registered engine by passing nil for the device
		for (engine = QADeviceGetFirstEngine(nil); engine; 
				 engine = QADeviceGetNextEngine(nil, engine)) {
			if (QAEngineGestalt(engine, kQAGestalt_ASCIINameLength, 
													&currentEngineNameLength) != kQANoErr) {
				continue;
			}
			
			if (currentEngineNameLength > kMaxEngineNameLength) {
	// should really allocate a temporary handle of the appropriate size
				continue;
			}
			
			if (QAEngineGestalt(engine, kQAGestalt_ASCIIName, 
													&currentEngineName) != kQANoErr) {
				continue;
			}
			
				// the menu requires Pascal strings
			S3CtoPString(currentEngineName);
			
				// add this engine name to the menu
			AppendSharedItem((StringPtr) currentEngineName);
		} // endfor
	}
}


// ===========================================================================
	void
CEnginePopupMenu::ReinitializeMenu()
{
	// first get rid of old data values
	// then call InitializeMenu()
	InitializeMenu();
}


// ===========================================================================
	void
CEnginePopupMenu::UpdatePopupMenus()
{
	if (!sDoUpdate) {
			// we're in the middle of a BeginUpdate/EndUpdate pair, so we don't
			// do the update until EndUpdate is called
		return;
	}
	
	LListIterator iterator(sPopupMenuList, iterate_FromStart);
	CEnginePopupMenu* thePopup = nil;

		// update the control values of each open popup
	while (iterator.Next(&thePopup)) {
		Assert_(thePopup);
		
		thePopup->UpdateValues();
	}
}


// ===========================================================================
	void
CEnginePopupMenu::BeginUpdate()
{
		// we set this flag to false so that UpdatePopupMenus() doesn't update
		// every popup after each call to AppendSharedItem, InsertSharedItem, etc.
		// when EndUpdate is called, this flag will be reset.
	sDoUpdate = false;
}


// ===========================================================================
	void
CEnginePopupMenu::EndUpdate()
{
		// turn updating back on
	sDoUpdate = true;
	
		// make sure all the popups get updated correctly, now that the caller
		// is done changing the shared menu.
	UpdatePopupMenus();
}


// ===========================================================================
// only one item can be appended at a time
//
	void
CEnginePopupMenu::AppendSharedItem(const Str255 inItemString /*, void* inData */)
{
	::AppendMenu(sMenuH, inItemString);
	
		// append the data for this item 
//	sDataList.InsertItemsAt(1, arrayIndex_Last, inData);
	
	UpdatePopupMenus();
}


// ===========================================================================
// only one item can be inserted at a time
//
	void
CEnginePopupMenu::InsertSharedItem(const Str255 inItemString, Int16 inAfterItem /*, void* inData */)
{
	::InsertMenuItem(sMenuH, inItemString, inAfterItem);
	
		// insert the data for this item.  inAfterItem is the index of the item
		// after which the new item is inserted, so add 1 to it get the LList index.
//	sDataList.InsertItemsAt(1, inAfterItem + 1, inData);
	
	UpdatePopupMenus();
}


// ===========================================================================
	void
CEnginePopupMenu::DeleteSharedItem(Int16 inItemNumber)
{
	::DeleteMenuItem(sMenuH, inItemNumber);
	
		// remove the item's corresponding data 
//	sDataList.RemoveItemsAt(1, inItemNumber);
	
	UpdatePopupMenus();
}


// ===========================================================================
	Int16
CEnginePopupMenu::GetSharedItemFromString(const Str255 inString)
{
		// Search for inString in the items of this Menu
	Str255 itemString;
	short numItems = ::CountMItems(sMenuH);
	
	for (short theIndex = 1; theIndex <= numItems; theIndex++) {
		::GetMenuItemText(sMenuH, theIndex, itemString);
		
		if (::EqualString(inString, itemString, true, false)) {
			return theIndex;
		}
	}
	
		// the string wasn't found in this menu
	return 0;
}


// ===========================================================================
	Int16
CEnginePopupMenu::GetSharedItemFromString(const char* inString)
{
	Str255 PString;
	S3CtoPString(inString, PString);
	
			// Search for inString in the items of this Menu
	Str255 itemString;
	short numItems = ::CountMItems(sMenuH);
	
	for (short theIndex = 1; theIndex <= numItems; theIndex++) {
		::GetMenuItemText(sMenuH, theIndex, itemString);
		
		if (::EqualString(PString, itemString, true, false)) {
			return theIndex;
		}
	}
	
		// the string wasn't found in this menu
	return 0;
}


// ===========================================================================
//  CEnginePopupMenu											   CEnginePopupMenu 
// ===========================================================================
	CEnginePopupMenu*
CEnginePopupMenu::CreateEnginePopupMenuStream(LStream	*inStream)
{
	return (new CEnginePopupMenu(inStream));
}


// ===========================================================================
//		 CEnginePopupMenu(LStream*)
// ===========================================================================
//	Construct from data in a Stream

CEnginePopupMenu::CEnginePopupMenu(
	LStream	*inStream)
		:	CSmartStdPopupMenu(inStream)
//		: CSharedPopupMenu(inStream)
{	
	InitEnginePopupMenu();
}


// ===========================================================================
//		 ~CEnginePopupMenu
// ===========================================================================
CEnginePopupMenu::~CEnginePopupMenu()
{	
		// remove us from the list of popup menus
	sPopupMenuList.Remove(&this);
	
		// get a handle to the popup's control record
	ControlHandle controlH = GetMacControl();
	
		// get a handle to the popup's private data record in the control record
	PopupPrivateDataHandle	dataH =
		(PopupPrivateDataHandle) (**controlH).contrlData;

		// we duplicate the menu and stick the copy in the control's private
		// data so that when ~LStdControl disposes it, the copy will be destroyed
		// and not the menu all the remaining versions of this popup are 
		// pointing to.
	::HandToHand((Handle*) &(**dataH).mHandle);
}


// ===========================================================================
//		 InitStringPopupMenu
// ===========================================================================
//	Private initializer
//
// LStdPopupMenu::InitStdPopupMenu() should handle setting the initial,
// min, and max values

	void
CEnginePopupMenu::InitEnginePopupMenu()
{	
		// add us to the list of existing popup menus.  we have to do this 
		// before initialzing the menu for the first time, because anything
		// that gets added to the menu won't show up in the very first popup
		// unless there's already a pointer to it in the sPopupMenuList.
	sPopupMenuList.InsertItemsAt(1, arrayIndex_Last, &this);
	
	if (sMenuH == nil) {
			// this is the first popup instance created, so we have to copy its
			// menu and dispose of the original.  if we don't, the menu shared
			// by all the popups in this class will be disposed when one of them 
			// is destroyed, so the remaining ones won't have any menu to use,
			// causing all sorts of errors.
		PopupPrivateDataHandle	dataH =
			(PopupPrivateDataHandle) (**GetMacControl()).contrlData;
	
		MenuHandle menuH = (**dataH).mHandle;
		MenuHandle menuCopyH = menuH;
		::HandToHand((Handle*) &menuCopyH);
		
			// dispose of the existing menu
		::ReleaseResource((Handle) menuH);

			// turn off the purgeable and resource bits in the menu handle we
			// just copied.  this may not be necessary, but it doesn't hurt anyting.
		::HNoPurge((Handle) menuCopyH);
		::HClrRBit((Handle) menuCopyH);

			// insert the copied menu as a popup
		::InsertMenu(menuCopyH, -1);

			// all the instances created after this will use this menu and ID
		sMenuH = menuCopyH;
		sMenuID = (**sMenuH).menuID;
		
			// make the menu handle of this popup point to the copied menu
		(**dataH).mHandle = sMenuH;

//		Int32 itemCount = GetMaxValue();
//		Int32 dataCount = sDataList.GetCount();
//		
//			// either of these two branches can be true only if this is the
//			// first popup menu to be created.  after that, the insertion and
//			// deletion methods should keep the data list and the menu in sync.
//		if (itemCount > dataCount) {
//				// the menu has more items than we have data values for, so append
//				// an empty FSSpec to the end of sDataList, one for each item that 
//				// has no data
//			FSSpec emptyFSSpec = { 0, 0, { 0 } };
//			sDataList.InsertItemsAt(itemCount - dataCount,
//														  arrayIndex_Last, &emptyFSSpec);
//		} else if (itemCount < dataCount) {
//				// the menu has fewer items than we have data values for, so remove
//				// the extra data values from the end of sDataList
//			sDataList.RemoveItemsAt(dataCount - itemCount, itemCount + 1);
//		}
		
			// we also have to initialize the menu data to hold the PICT file specs
		InitializeMenu();
	}
	
	 // save the string of the item currently selected in the popup
	GetStringFromItem(GetValue(), mValueString);

		// broadcast the current item's string 
	SetBroadcastString(true);
}


// ===========================================================================
	void
CEnginePopupMenu::SetValue(Int32 inValue)
{
	inherited::SetValue(inValue);
	
		// save the string of the item we just set
	GetStringFromItem(GetValue(), mValueString);
}


// ===========================================================================
	void
CEnginePopupMenu::AppendItem(const Str255 inItemString)
{
	AppendSharedItem(inItemString);
}


// ===========================================================================
	void
CEnginePopupMenu::InsertItem(const Str255 inItemString, Int16 inAfterItem)
{
	InsertSharedItem(inItemString, inAfterItem);
}


// ===========================================================================
	void
CEnginePopupMenu::DeleteItem(Int16 inItemNumber)
{
	DeleteSharedItem(inItemNumber);
}


// ===========================================================================
	void
CEnginePopupMenu::UpdateValues()
{
		// determine the maximum value by counting the number of items in 
		// the menu resource
	SetMaxValue(::CountMItems(GetMacMenuH()));
	
	if (GetMaxValue() > 0) {
			// there's at least one item in the menu, so make sure the min
			// value is 1, since we might have just added the first item in the
			// menu, and the min value might still be 0
		SetMinValue(1);
		
			// if items in the menu were changed, the text of the item with 
			// the same number as the value of the popup might be different 
			// than what it was before, because the order of the items has
			// shifted, or the previously selected item was deleted.  although
			// the popup's value is still the same, to the user it won't look 
			// like the same item is selected since a different string appears
			// next to the title.  so reset the value according to the string
			// of the item that was previously selected.  if the item has been
			// deleted (and thus its string can't be found in the menu), 
			// SetValueByString will default to item 1.
		SetValueByString(mValueString);
	} else {
			// there are no items in the menu, so set the other values to
			// reflect that
		SetMinValue(0);
		SetValue(0);
	}
}
