// ===========================================================================
//	CSettingsWindow.cp
// ===========================================================================


#include <QuickDraw.h>
#include <OSUtils.h>
#include <LDocument.h>

#include "RAVE.h"
#include "CTestInfo.h"
#include "CSettingsWindow.h"
#include "S3Utilities.h"
#include "S3Alert.h"


Boolean CSettingsWindow::sUpdateSettingsWindows = true;


// ===========================================================================
//		 CreateSettingsWindow
// ===========================================================================
//	Return a newly created Window object initialized from a PPob resource

	CSettingsWindow*
CSettingsWindow::CreateSettingsWindow(
	ResIDT		inWindowID,
	LCommander	*inSuperCommander)
{
	SetDefaultCommander(inSuperCommander);

	CSettingsWindow	*theWindow =
			(CSettingsWindow*) UReanimator::ReadObjects('PPob', inWindowID);

	theWindow->FinishCreate();

	if (theWindow->HasAttribute(windAttr_ShowNew)) {
		theWindow->Show();
	}
	
	return theWindow;
}


// ===========================================================================
//		 CreateSettingsWindowStream
// ===========================================================================
//	Return a newly created Window object initialized with data from a Stream

	CSettingsWindow*
CSettingsWindow::CreateSettingsWindowStream(
	LStream	*inStream)
{
	return (new CSettingsWindow(inStream));
}


// ===========================================================================
//		 CSettingsWindow
// ===========================================================================
//	Default Constructor

CSettingsWindow::CSettingsWindow()
	: LWindow()
{
}


// ===========================================================================
//		 CSettingsWindow(SWindowInfo&)
// ===========================================================================
//	Construct Window from the data in a struct

CSettingsWindow::CSettingsWindow(
	const SWindowInfo	&inWindowInfo)
	: LWindow(inWindowInfo)
{
}


// ===========================================================================
//		 CSettingsWindow(ResIDT, Uint32, LCommander*)
// ===========================================================================
//	Construct a Window from a WIND Resource with the specified attributes
//	and SuperCommander

CSettingsWindow::CSettingsWindow(
	ResIDT		inWINDid,
	Uint32		inAttributes,
	LCommander	*inSuper)
		: LWindow(inWINDid, inAttributes, inSuper)
{
}


// ===========================================================================
//		 CSettingsWindow(LStream*)
// ===========================================================================
//	Construct Window from the data in a stream

CSettingsWindow::CSettingsWindow(
	LStream	*inStream)
	: LWindow(inStream)
{
}


// ===========================================================================
	void
CSettingsWindow::Draw(
	RgnHandle	inSuperDrawRgnH)
{
	if (sUpdateSettingsWindows) {
			// we should update the window, so let LWindow do its thing
		LWindow::Draw(inSuperDrawRgnH);
	}
}


// ===========================================================================
	void
CSettingsWindow::ClickInContent(
	const EventRecord	&inMacEvent)
{
		// let the window handle the click first
	LWindow::ClickInContent(inMacEvent);
	
	if (inMacEvent.modifiers & cmdKey) {
		::SysBeep(30);
		
			// toggle the update flag
		sUpdateSettingsWindows = !sUpdateSettingsWindows;
		
		if (sUpdateSettingsWindows) {
				// the flag has just been turned back on, so update the entire window
			Draw(nil);
		}
	}
}
