// ===========================================================================
//	CSharedPopupMenu.cp
// ===========================================================================


#include <LListIterator.h>

#include "CSharedPopupMenu.h"
#include "StringConversion.h"


MenuHandle CSharedPopupMenu::sMenuH = nil;
ResIDT CSharedPopupMenu::sMenuID = 0;
LList CSharedPopupMenu::sPopupMenuList;
Boolean CSharedPopupMenu::sDoUpdate = true;


// ===========================================================================
	void
CSharedPopupMenu::UpdatePopupMenus()
{
	if (!sDoUpdate) {
			// we're in the middle of a BeginUpdate/EndUpdate pair, so we don't
			// do the update until EndUpdate is called
		return;
	}
	
	LListIterator iterator(sPopupMenuList, iterate_FromStart);
	CSharedPopupMenu* thePopup = nil;

		// update the control values of each open popup
	while (iterator.Next(&thePopup)) {
		Assert_(thePopup);
		
		thePopup->UpdateValues();
	}
}


// ===========================================================================
	void
CSharedPopupMenu::BeginUpdate()
{
		// we set this flag to false so that UpdatePopupMenus() doesn't update
		// every popup after each call to AppendSharedItem, InsertSharedItem, etc.
		// when EndUpdate is called, this flag will be reset.
	sDoUpdate = false;
}


// ===========================================================================
	void
CSharedPopupMenu::EndUpdate()
{
		// turn updating back on
	sDoUpdate = true;
	
		// make sure all the popups get updated correctly, now that the caller
		// is done changing the shared menu.
	UpdatePopupMenus();
}


// ===========================================================================
	void
CSharedPopupMenu::AppendSharedItem(const Str255 inItemString)
{
	::AppendMenu(sMenuH, inItemString);
	
		// the number of the item we just appended is the total number of items
		// in the menu
	short appendedItem = ::CountMItems(sMenuH);
	
		// we set the text of the item we just appended to inItemString
		// again, just in case the string contains any metacharacters
		// that control the item's formatting.  this means we can't append
		// any items with formatting, but that's usually what we want when
		// appending strings like filenames, which often contain chars like '('
	::SetMenuItemText(sMenuH, appendedItem, inItemString);
	
		// now make sure the item is enabled, in case its text contains '(',
		// which would have disabled it during the AppendMenu call.
	::EnableItem(sMenuH, appendedItem);
	
	UpdatePopupMenus();
}


// ===========================================================================
	void
CSharedPopupMenu::InsertSharedItem(const Str255 inItemString, Int16 inAfterItem)
{
	::InsertMenuItem(sMenuH, inItemString, inAfterItem);
	
		// the number of the item we just inserted is the one 
		// following inAfterItem
	short insertedItem = inAfterItem + 1;
	
		// we set the text of the item we just inserted to inItemString
		// again, just in case the string contains any metacharacters
		// that control the item's formatting.  this means we can't insert
		// any items with formatting, but that's usually what we want when
		// inserting strings like filenames, which often contain chars like '('
	::SetMenuItemText(sMenuH, insertedItem, inItemString);
	
		// now make sure the item is enabled, in case its text contains '(',
		// which would have disabled it during the AppendMenu call.
	::EnableItem(sMenuH, insertedItem);
	
	UpdatePopupMenus();
}


// ===========================================================================
	void
CSharedPopupMenu::AppendSharedResourceItems(ResType inResType)
{
	::AppendResMenu(sMenuH, inResType);
	
	UpdatePopupMenus();
}


// ===========================================================================
	void
CSharedPopupMenu::InsertSharedResourceItems(ResType inResType, Int16 inAfterItem)
{
	::InsertResMenu(sMenuH, inResType, inAfterItem);
	
	UpdatePopupMenus();
}


// ===========================================================================
	void
CSharedPopupMenu::DeleteSharedItem(Int16 inItemNumber)
{
	::DeleteMenuItem(sMenuH, inItemNumber);
	
	UpdatePopupMenus();
}


// ===========================================================================
	Int16
CSharedPopupMenu::GetSharedItemFromString(const Str255 inString) const
{
		// Search for inString in the items of this Menu
	Str255 itemString;
	short numItems = ::CountMItems(sMenuH);
	
	for (short theIndex = 1; theIndex <= numItems; theIndex++) {
		::GetMenuItemText(sMenuH, theIndex, itemString);
		
		if (::EqualString(inString, itemString, true, false)) {
			return theIndex;
		}
	}
	
		// the string wasn't found in this menu
	return 0;
}


// ===========================================================================
	Int16
CSharedPopupMenu::GetSharedItemFromString(const char* inString) const
{
	Str255 PString;
	S3CtoPString(inString, PString);
	
			// Search for inString in the items of this Menu
	Str255 itemString;
	short numItems = ::CountMItems(sMenuH);
	
	for (short theIndex = 1; theIndex <= numItems; theIndex++) {
		::GetMenuItemText(sMenuH, theIndex, itemString);
		
		if (::EqualString(PString, itemString, true, false)) {
			return theIndex;
		}
	}
	
		// the string wasn't found in this menu
	return 0;
}


// ===========================================================================
//  CSharedPopupMenu											   CSharedPopupMenu 
// ===========================================================================
	CSharedPopupMenu*
CSharedPopupMenu::CreateSharedPopupMenuStream(LStream	*inStream)
{
	return (new CSharedPopupMenu(inStream));
}


// ===========================================================================
//		 CSharedPopupMenu
// ===========================================================================
//	Construct from input parameters

CSharedPopupMenu::CSharedPopupMenu(
	const SPaneInfo	&inPaneInfo,
	MessageT		inValueMessage,
	Int16			inTitleOptions,
	ResIDT			inMENUid,
	Int16			inTitleWidth,
	Int16			inPopupVariation,
	ResIDT			inTextTraitsID,
	Str255			inTitle,
	OSType			inResTypeMENU,
	Int16			inInitialMenuItem)
		: CSmartStdPopupMenu(inPaneInfo, inValueMessage, inTitleOptions,
				inMENUid, inTitleWidth, inPopupVariation, inTextTraitsID,
				inTitle, inResTypeMENU, inInitialMenuItem)
{
	InitSharedPopupMenu();
}


// ===========================================================================
//		 CSharedPopupMenu
// ===========================================================================
//	Construct from input parameters and an existing ControlHandle

CSharedPopupMenu::CSharedPopupMenu(
	const SPaneInfo	&inPaneInfo,
	MessageT		inValueMessage,
	Int32			inMaxValue,
	ResIDT			inTextTraitsID,
	ControlHandle	inMacControlH)
		: CSmartStdPopupMenu(inPaneInfo, inValueMessage, inMaxValue,
						inTextTraitsID, inMacControlH)
{
	InitSharedPopupMenu();
}


// ===========================================================================
//		 CSharedPopupMenu(LStream*)
// ===========================================================================
//	Construct from data in a Stream

CSharedPopupMenu::CSharedPopupMenu(
	LStream	*inStream)
		: CSmartStdPopupMenu(inStream)
{	
	InitSharedPopupMenu();
}


// ===========================================================================
//		 ~CSharedPopupMenu
// ===========================================================================
CSharedPopupMenu::~CSharedPopupMenu()
{	
		// remove us from the list of popup menus
	sPopupMenuList.Remove(&this);
	
		// get a handle to the popup's control record
	ControlHandle controlH = GetMacControl();
	
		// get a handle to the popup's private data record in the control record
	PopupPrivateDataHandle	dataH =
		(PopupPrivateDataHandle) (**controlH).contrlData;

MenuHandle menuH = (**dataH).mHandle;
MenuPtr menuP = *menuH;
char handleState = ::HGetState((Handle) (**dataH).mHandle);
		
		// make the menu handle nil, so that 
	(**dataH).mHandle = nil;
	(**dataH).mID = 9999;
	
		// dispose of the data record so that DisposeControl can't get ahold
		// of it and dispose of the menu handle it contains
//	::DisposeHandle((Handle) dataH);
	
		// set the control's refCon to 0, so that DisposeControl doesn't see
		// anything there to dispose of
//	::SetControlReference(controlH, 0);
}


// ===========================================================================
//		 InitSharedPopupMenu
// ===========================================================================
//	Private initializer

	void
CSharedPopupMenu::InitSharedPopupMenu()
{
	if (sMenuH == nil) {
			// this is the first popup instance created, so we have to set the 
			// sMenuH class variable to the menu handle created for us by PowerPlant
			// so that all subsequent instances share the same menu
		sMenuH = inherited::GetMacMenuH();
		sMenuID = (**sMenuH).menuID;
	}
	
		// get a handle to the popup's private data record in the control record
	PopupPrivateDataHandle	dataH =
		(PopupPrivateDataHandle) (**GetMacControl()).contrlData;

		// make the menu handle of this popup point to the static menu shared
		// by all instances of this class
	(**dataH).mHandle = sMenuH;

		// add us to the list of existing popup menus
	sPopupMenuList.InsertItemsAt(1, arrayIndex_Last, &this);
	
	 // save the string of the item currently selected in the popup
	GetStringFromItem(GetValue(), mValueString);
}


// ===========================================================================
	void
CSharedPopupMenu::SetValue(Int32 inValue)
{
	inherited::SetValue(inValue);
	
		// save the string of the item we just set
	GetStringFromItem(GetValue(), mValueString);
}


// ===========================================================================
	void
CSharedPopupMenu::AppendItem(const Str255 inItemString)
{
	AppendSharedItem(inItemString);
}


// ===========================================================================
	void
CSharedPopupMenu::InsertItem(const Str255 inItemString, Int16 inAfterItem)
{
	InsertSharedItem(inItemString, inAfterItem);
}


// ===========================================================================
	void
CSharedPopupMenu::AppendResourceItems(ResType inResType)
{
	AppendSharedResourceItems(inResType);
}


// ===========================================================================
	void
CSharedPopupMenu::InsertResourceItems(ResType inResType, Int16 inAfterItem)
{
	InsertSharedResourceItems(inResType, inAfterItem);
}


// ===========================================================================
	void
CSharedPopupMenu::DeleteItem(Int16 inItemNumber)
{
	DeleteSharedItem(inItemNumber);
}


// ===========================================================================
	void
CSharedPopupMenu::UpdateValues()
{
		// determine the maximum value by counting the number of items in 
		// the menu resource
	SetMaxValue(::CountMItems(GetMacMenuH()));
	
	if (GetMaxValue() > 0) {
			// there's at least one item in the menu, so make sure the min
			// value is 1, since we might have just added the first item in the
			// menu, and the min value might still be 0
		SetMinValue(1);
		
			// if items in the menu were changed, the text of the item with 
			// the same number as the value of the popup might be different 
			// than what it was before, because the order of the items has
			// shifted, or the previously selected item was deleted.  although
			// the popup's value is still the same, to the user it won't look 
			// like the same item is selected since a different string appears
			// next to the title.  so reset the value according to the string
			// of the item that was previously selected.  if the item has been
			// deleted (and thus its string can't be found in the menu), 
			// SetValueByString will default to item 1.
		SetValueByString(mValueString);
	} else {
			// there are no items in the menu, so set the other values to
			// reflect that
		SetMinValue(0);
		SetValue(0);
	}
}

