// ===========================================================================
//	CSmartStdPopupMenu.cp
// ===========================================================================
//
// This derivative of a standard popup menu has methods for inserting and
// appending menu items which automatically update the menu's min and max
// values.  You shouldn't call functions like AppendMenu directly on the
// Mac menu handle.


#ifdef PowerPlant_PCH
#include PowerPlant_PCH
#endif


#include <PP_Messages.h>

#include "CSmartStdPopupMenu.h"
#include "StringConversion.h"


// ===========================================================================
//  CSmartStdPopupMenu											   CSmartStdPopupMenu 
// ===========================================================================
	CSmartStdPopupMenu*
CSmartStdPopupMenu::CreateSmartStdPopupMenuStream(LStream	*inStream)
{
	return (new CSmartStdPopupMenu(inStream));
}


// ===========================================================================
//		 CSmartStdPopupMenu
// ===========================================================================
//	Construct from input parameters

CSmartStdPopupMenu::CSmartStdPopupMenu(
	const SPaneInfo	&inPaneInfo,
	MessageT		inValueMessage,
	Int16			inTitleOptions,
	ResIDT			inMENUid,
	Int16			inTitleWidth,
	Int16			inPopupVariation,
	ResIDT			inTextTraitsID,
	Str255			inTitle,
	OSType			inResTypeMENU,
	Int16			inInitialMenuItem)
		: LStdPopupMenu(inPaneInfo, inValueMessage, inTitleOptions,
				inMENUid, inTitleWidth, inPopupVariation, inTextTraitsID,
				inTitle, inResTypeMENU, inInitialMenuItem)
{
	InitSmartStdPopupMenu();
}


// ===========================================================================
//		 CSmartStdPopupMenu
// ===========================================================================
//	Construct from input parameters and an existing ControlHandle

CSmartStdPopupMenu::CSmartStdPopupMenu(
	const SPaneInfo	&inPaneInfo,
	MessageT		inValueMessage,
	Int32			inMaxValue,
	ResIDT			inTextTraitsID,
	ControlHandle	inMacControlH)
		: LStdPopupMenu(inPaneInfo, inValueMessage, inMaxValue,
						inTextTraitsID, inMacControlH)
{
	InitSmartStdPopupMenu();
}


// ===========================================================================
//		 CSmartStdPopupMenu(LStream*)
// ===========================================================================
//	Construct from data in a Stream

CSmartStdPopupMenu::CSmartStdPopupMenu(
	LStream	*inStream)
		: LStdPopupMenu(inStream)
{	
//	mMaxValue=4+StringWidth(sMenuTitle);	//	Size of the Control Title string.

	InitSmartStdPopupMenu();
}


// ===========================================================================
//		 InitSmartStdPopupMenu
// ===========================================================================
//	Private initializer
//
// LStdPopupMenu::InitStrPopupMenu() should handle setting the initial,
// min, and max values

	void
CSmartStdPopupMenu::InitSmartStdPopupMenu()
{
		// this class's GetMacMenuH() is inlined and just returns mMenuH, so
		// make sure mMenuH is valid by calling our inherited GetMacMenuH()
	mMenuH = inherited::GetMacMenuH();

		// by default, don't broadcast strings as our value
	mBroadcastString = false;
}


// ===========================================================================
	void
CSmartStdPopupMenu::BroadcastValueMessage()
{
	if (mValueMessage != cmd_Nothing) {
		if (mBroadcastString) {
			Str255 theString;
			GetString(theString);
			
			BroadcastMessage(mValueMessage, (void*) theString);
		} else {
			Int32	value = GetValue();

			BroadcastMessage(mValueMessage, (void*) &value);
		}
	}
}


// ===========================================================================
	void
CSmartStdPopupMenu::GetString(Str255 outString) const
{	
	GetStringFromItem((Int16) GetValue(), outString);
}


// ===========================================================================
	void
CSmartStdPopupMenu::GetString(char* outString) const
{	
	GetStringFromItem((Int16) GetValue(), outString);
}


// ===========================================================================
	void
CSmartStdPopupMenu::SetValueByString(Str255 inString)
{	
	Int16 index = GetItemFromString(inString);
	
	if (index == 0) {
			// this string wasn't found in the menu, so default to the first item
		index = 1;
	}
	
	SetValue(index);
}


// ===========================================================================
	void
CSmartStdPopupMenu::SetValueByString(char* inString)
{	
	Int16 index = GetItemFromString(inString);
	
	if (index == 0) {
			// this string wasn't found in the menu, so default to the first item
		index = 1;
	}
	
	SetValue(index);
}


// ===========================================================================
//		 GetStringFromItem
// ===========================================================================
//	Return the value for a particular Menu item

	void
CSmartStdPopupMenu::GetStringFromItem(Int16 inIndex, Str255 outString) const
{
	if (inIndex > GetMaxValue() || inIndex < 1) {
			// the index is out of range, so make the string empty
		outString[0] = 0;
		
		return;
	}
	
	::GetMenuItemText(GetMacMenuH(), inIndex, outString);
}


// ===========================================================================
//		 GetStringFromItem
// ===========================================================================
//	Return the value for a particular Menu item

	void
CSmartStdPopupMenu::GetStringFromItem(Int16 inIndex, char* outString) const
{
	if (inIndex > GetMaxValue() || inIndex < 1) {
			// the index is out of range, so make the string empty
		outString = nil;
		
		return;
	}
	
	::GetMenuItemText(GetMacMenuH(), inIndex, (StringPtr) outString);
	
		// convert the Pascal string to a C string in place
	S3PtoCString((unsigned char*) outString);
//	S3PtoCString((unsigned char*) outString, StrLength((StringPtr) outString));
}


// ===========================================================================
//		 GetItemFromString
// ===========================================================================
//	Return the Menu item index number for a particular value
//		Return 0 if the value number is not used for this Menu

	Int16
CSmartStdPopupMenu::GetItemFromString(const Str255 inString) const
{
		// Search for inString in the items of this Menu
	Str255 itemString;
	short numItems = GetMaxValue();
	
	for (short theIndex = 1; theIndex <= numItems; theIndex++) {
		::GetMenuItemText(GetMacMenuH(), theIndex, itemString);
		
		if (::EqualString(inString, itemString, true, false)) {
			return theIndex;
		}
	}
	
		// the string wasn't found in this menu
	return 0;
}


// ===========================================================================
//		 GetItemFromString
// ===========================================================================
//	Return the Menu item index number for a particular value
//		Return 0 if the value number is not used for this Menu

	Int16
CSmartStdPopupMenu::GetItemFromString(const char* inString) const
{
	Str255 PString;
	S3CtoPString(inString, PString);
	
			// Search for inString in the items of this Menu
	Str255 itemString;
	short numItems = GetMaxValue();
	
	for (short theIndex = 1; theIndex <= numItems; theIndex++) {
		::GetMenuItemText(GetMacMenuH(), theIndex, itemString);
		
		if (::EqualString(PString, itemString, true, false)) {
			return theIndex;
		}
	}
	
		// the string wasn't found in this menu
	return 0;
}


// ===========================================================================
	void
CSmartStdPopupMenu::AppendItem(const Str255 inItemString)
{
	::AppendMenu(GetMacMenuH(), inItemString);
	
	UpdateValues();
}


// ===========================================================================
	void
CSmartStdPopupMenu::InsertItem(const Str255 inItemString, Int16 inAfterItem)
{
	::InsertMenuItem(GetMacMenuH(), inItemString, inAfterItem);
	
	UpdateValues();
}

// ===========================================================================
	void
CSmartStdPopupMenu::AppendResourceItems(ResType inResType)
{
	::AppendResMenu(GetMacMenuH(), inResType);
	
	UpdateValues();
}


// ===========================================================================
	void
CSmartStdPopupMenu::InsertResourceItems(ResType inResType, Int16 inAfterItem)
{
	::InsertResMenu(GetMacMenuH(), inResType, inAfterItem);
	
	UpdateValues();
}


// ===========================================================================
	void
CSmartStdPopupMenu::DeleteItem(Int16 inItemNumber)
{
	::DeleteMenuItem(GetMacMenuH(), inItemNumber);
	
	UpdateValues();
}


// ===========================================================================
	void
CSmartStdPopupMenu::UpdateValues()
{
		// determine the maximum value by counting the number of items in 
		// the menu resource
	SetMaxValue(::CountMItems(GetMacMenuH()));
	
	if (GetMaxValue() > 0) {
			// there's at least one item in the menu, so make sure the min
			// value is 1, since we might have just added the first item in the
			// menu, and the min value might still be 0
		SetMinValue(1);
		
		if (GetValue() < 1) {
				// this menu previously had no item selected (value == 0), so
				// select the first item
			SetValue(1);
		}
	} else {
			// there are no items in the menu, so set the other values to
			// reflect that
		SetMinValue(0);
		SetValue(0);
	}
}

