// ===========================================================================
//	CSmartTextEdit.cp
// ===========================================================================


#include <LCommander.h>

#include "CSmartTextEdit.h"


// ---------------------------------------------------------------------------
//		 CreateSmartTextEditStream
// ---------------------------------------------------------------------------
//	Create a new TextEdit object from the data in a Stream

CSmartTextEdit*
CSmartTextEdit::CreateSmartTextEditStream(
	LStream	*inStream)
{
	return (new CSmartTextEdit(inStream));
}


// ---------------------------------------------------------------------------
//		 CSmartTextEdit
// ---------------------------------------------------------------------------
//	Default Contructor

CSmartTextEdit::CSmartTextEdit()
		: LTextEdit()
{
	mIsDirty = false;
}


// ---------------------------------------------------------------------------
//		 CSmartTextEdit
// ---------------------------------------------------------------------------
//	Construct from input parameters

CSmartTextEdit::CSmartTextEdit(
	const SPaneInfo	&inPaneInfo,
	const SViewInfo	&inViewInfo,
	Uint16			inTextAttributes,
	ResIDT			inTextTraitsID)
		: LTextEdit(inPaneInfo, inViewInfo, inTextAttributes, inTextTraitsID)
{
	mIsDirty = false;
}


// ---------------------------------------------------------------------------
//		 CSmartTextEdit(LStream*)
// ---------------------------------------------------------------------------
//	Contruct an TextEdit from the data in a Stream

CSmartTextEdit::CSmartTextEdit(
	LStream	*inStream)
		: LTextEdit(inStream)
{
	mIsDirty = false;
}


// ---------------------------------------------------------------------------
//		 HandleKeyPress
// ---------------------------------------------------------------------------
//	Handle key stroke directed at an TextEdit
//
//	Return true if the TextEdit handles the keystroke

	Boolean
CSmartTextEdit::HandleKeyPress(
	const EventRecord&	inKeyEvent)
{
	if (!HasAttribute(textAttr_Editable)) {
			// this TextEdit view is not editable, so let our commander handle
			// the key press
		return LCommander::HandleKeyPress(inKeyEvent);
	} else {
		return inherited::HandleKeyPress(inKeyEvent);
	}
}


// ===========================================================================
	void
CSmartTextEdit::AppendText(char* inText, long inTextLength)
{
	InsertText(inText, inTextLength, TextPosition_End);

		// scroll the text area to the position at which we inserted the new text.
	SDimension16	frameSize;
	SDimension32	imageSize;
	GetFrameSize(frameSize);
	GetImageSize(imageSize);

	if (imageSize.height > frameSize.height) {
			// we leave the horizontal scroll bar at the far-left position and move 
			// vertically to the bottom of the output.  we do this only if there is
			// something to scroll to (imageSize.height > frameSize.height).  
			// otherwise, the test output in a new window would appear at the bottom 
			// of the test area.
		ScrollImageTo(0, imageSize.height - frameSize.height, true);
	}
}



// ===========================================================================
	void
CSmartTextEdit::InsertText(char* inText, long inTextLength, long inPosition)
{
	if (inTextLength < 1) {
			// there's nothing to insert, so just return
		return;
	}
	
		// make sure we're drawing into the TE pane
	FocusDraw();
	
	if (inPosition != -1) {
			// the caller explicitly passed the position at which the text 
			// should be inserted, so put the insertion point there before
			// inserting anything
		::TESetSelect(inPosition, inPosition, mTextEditH);
	}
	
		// insert the text into the TE handle at the current position
	::TEInsert(inText, inTextLength, mTextEditH);
	
		// since we just inserted some text, the text view is dirty
	mIsDirty = true;
	
		// make sure the text image corresponds to what's in the TextEdit handle
	AdjustImageToText();

//		// scroll the text area to the position at which we inserted the new text.
//	SDimension16	frameSize;
//	SDimension32	imageSize;
//	GetFrameSize(frameSize);
//	GetImageSize(imageSize);
//	
//		// we leave the horizontal scroll bar at the far-left position and move 
//		// vertically to the insertion point
//	ScrollImageTo(0, inPosition, true);
//	Draw(nil);
////	ScrollImageTo(0, imageSize.height - frameSize.height, true);
}


// ===========================================================================
	void
CSmartTextEdit::UserChangedText()
{	
		// the user just chagned the text, so we're now dirty.  
	if (!mIsDirty) {
			// call SetUpdateCommandStatus to change the File menu so that the 
			// Save and Revert commands are now enabled.
		SetUpdateCommandStatus(true);
		
		mIsDirty = true;
	}
}
