// ===========================================================================
//	CSneak3viewApp.cp
// ===========================================================================


#include <StandardFile.h>

#include <LApplication.h>
#include <LGrowZone.h>
#include <LWindow.h>
#include <LScroller.h>
#include <LTextEdit.h>
#include <LPrintout.h>
#include <LPlaceHolder.h>
#include <LGroupBox.h>
#include <LPicture.h>

#include <UMemoryMgr.h>
#include <UDrawingState.h>
#include <URegistrar.h>
#include <UDesktop.h>

#include "CSneak3viewApp.h"
#include "S3Messages.h"
#include "CSettingsDoc.h"
#include "CSettingsWindow.h"
#include "CComparisonWindow.h"
#include "CSmartStdPopupMenu.h"
#include "CSharedPopupMenu.h"
#include "CPvalPopupMenu.h"
#include "CEnginePopupMenu.h"
#include "CPICTPopupMenu.h"
#include "CCLuTPopupMenu.h"
#include "CTestPopupMenu.h"
#include "CBroadcastEditField.h"
#include "CSmartTextEdit.h"
#include "CCommandButton.h"
#include "CDottedLine.h"
#include "CAboutBox.h"
#include "CEnablerView.h"
#include "LTab.h"
#include "LTabViewAlternator.h"
#include "LCommanderView.h"
#include "S3TestSystem.h"


// ===========================================================================
// 	Constants
// ===========================================================================

const ResIDT	WIND_AboutBox		= 4000;


// ===========================================================================
// 	Static
// ===========================================================================

CSneak3viewApp* CSneak3viewApp::sInstance = nil;


// ===========================================================================
//		 Main Program
// ===========================================================================

void main(void)
{
									// Set Debugging options
	SetDebugThrow_(debugAction_Alert);
	SetDebugSignal_(debugAction_Alert);

	InitializeHeap(3);				// Initialize Memory Manager
									// Parameter is number of Master Pointer
									//   blocks to allocate
	
									// Initialize standard Toolbox managers
	UQDGlobals::InitializeToolbox(&qd);	
	
	new LGrowZone(20000);			// Install a GrowZone function to catch
									//    low memory situations.
									//    Parameter is size of reserve memory
									//    block to allocated. The first time
									//    the GrowZone function is called,
									//    there will be at least this much
									//    memory left (so you'll have enough
									//    memory to alert the user or finish
									//    what you are doing).
	
	CSneak3viewApp	theApp;				// Create instance of your Application
	theApp.Run();					//   class and run it
}


// ===========================================================================
//		 CSneak3viewApp Class
// ===========================================================================


// ===========================================================================
	CSneak3viewApp*
CSneak3viewApp::Instance()
{
	if (sInstance == nil) {
		sInstance = new CSneak3viewApp;
	}
	
	return sInstance;
}


// ---------------------------------------------------------------------------
//		 CSneak3viewApp
// ---------------------------------------------------------------------------
//	Constructor

CSneak3viewApp::CSneak3viewApp()
{
	sInstance = this;
	
		// Register classes for objects created from 'PPob' resources
	URegistrar::RegisterClass(LWindow::class_ID,		LWindow::CreateWindowStream);
	URegistrar::RegisterClass(LScroller::class_ID,		LScroller::CreateScrollerStream);
	URegistrar::RegisterClass(LPlaceHolder::class_ID,	LPlaceHolder::CreatePlaceHolderStream);
	URegistrar::RegisterClass(LPrintout::class_ID,		LPrintout::CreatePrintoutStream);
	URegistrar::RegisterClass(LView::class_ID,			LView::CreateViewStream);
	URegistrar::RegisterClass(LPicture::class_ID,		LPicture::CreatePictureStream);
	URegistrar::RegisterClass(LDialogBox::class_ID,		LDialogBox::CreateDialogBoxStream);

	URegistrar::RegisterClass(LStdButton::class_ID, LStdButton::CreateStdButtonStream);
	URegistrar::RegisterClass(LStdCheckBox::class_ID, LStdCheckBox::CreateStdCheckBoxStream);
	URegistrar::RegisterClass(LStdRadioButton::class_ID, LStdRadioButton::CreateStdRadioButtonStream);
	URegistrar::RegisterClass(LStdPopupMenu::class_ID, LStdPopupMenu::CreateStdPopupMenuStream);
	URegistrar::RegisterClass(LButton::class_ID, LButton::CreateButtonStream);
	URegistrar::RegisterClass(LCaption::class_ID, LCaption::CreateCaptionStream);
	URegistrar::RegisterClass(LRadioGroup::class_ID, LRadioGroup::CreateRadioGroupStream);
	URegistrar::RegisterClass(LGroupBox::class_ID, LGroupBox::CreateGroupBoxStream);

	URegistrar::RegisterClass(LEditField::class_ID, LEditField::CreateEditFieldStream);
	URegistrar::RegisterClass(LTabGroup::class_ID, LTabGroup::CreateTabGroupStream);
	URegistrar::RegisterClass(LTextEdit::class_ID, LTextEdit::CreateTextEditStream);

	URegistrar::RegisterClass(CSmartStdPopupMenu::class_ID, CSmartStdPopupMenu::CreateSmartStdPopupMenuStream);
	URegistrar::RegisterClass(CPvalPopupMenu::class_ID, CPvalPopupMenu::CreatePvalPopupMenuStream);
	URegistrar::RegisterClass(CEnginePopupMenu::class_ID, CEnginePopupMenu::CreateEnginePopupMenuStream);
	URegistrar::RegisterClass(CBroadcastEditField::class_ID, CBroadcastEditField::CreateBroadcastEditFieldStream);
	URegistrar::RegisterClass(CTinselTestWindow::class_ID, CTinselTestWindow::CreateTinselTestWindowStream);
	URegistrar::RegisterClass(CSmartTextEdit::class_ID, CSmartTextEdit::CreateSmartTextEditStream);
	URegistrar::RegisterClass(CSharedPopupMenu::class_ID, CSharedPopupMenu::CreateSharedPopupMenuStream);
//	URegistrar::RegisterClass(CSharedDataPopupMenu<FSSpec, CTexturePopupMenuInitializer>::class_ID, CSharedDataPopupMenu<FSSpec, CTexturePopupMenuInitializer>::CreateFromStream);
	URegistrar::RegisterClass(CPICTPopupMenu::class_ID, CPICTPopupMenu::CreatePICTPopupMenuStream);
	URegistrar::RegisterClass(CCLuTPopupMenu::class_ID, CCLuTPopupMenu::CreateCLuTPopupMenuStream);
	URegistrar::RegisterClass(CDottedLine::class_ID, CDottedLine::CreateDottedLineStream);
	URegistrar::RegisterClass(CTestPopupMenu::class_ID, CTestPopupMenu::CreateTestPopupMenuStream);
	URegistrar::RegisterClass(CCommandButton::class_ID, CCommandButton::CreateCommandButtonStream);
	URegistrar::RegisterClass(CSettingsWindow::class_ID, CSettingsWindow::CreateSettingsWindowStream);
	URegistrar::RegisterClass(CComparisonWindow::class_ID, CComparisonWindow::CreateComparisonWindowStream);
	URegistrar::RegisterClass(LTab::class_ID, LTab::CreateTabStream);
	URegistrar::RegisterClass(LTabViewAlternator::class_ID, LTabViewAlternator::CreateTabViewAlternatorStream);
	URegistrar::RegisterClass(LCommanderView::class_ID, LCommanderView::CreateCommanderViewStream);
	URegistrar::RegisterClass(CAboutBox::class_ID, CAboutBox::CreateAboutBoxStream);
	URegistrar::RegisterClass(CEnablerView::class_ID, CEnablerView::CreateEnablerViewStream);

		// we show a splash screen while the prefs file (or the double-clicked
		// startup documents) is loaded and its window created, since this 
		// might take a few seconds.  the first time the texture popup menu is 
		// created, it has to look in the entire Sneak3view folder hierarchy for 
		// PICTs.  the splash screen is hidden in ShowTestWindows, which is 
		// called after all the documents have been opened after startup.  we
		// have to open here, in the creator, because StartUp doesn't get called
		// if the app is launched by double-clicking on one of its documents.
	CAboutBox::ShowSplashScreen(this);
	
		// we initialize these members here rather than in StartUp because
		// we want them to be initialized even if the app is opened by
		// double-clicking a document in the Finder.
	mLeftTestWindow = nil;
	mRightTestWindow = nil;
	CreateTestWindows();
}


// ---------------------------------------------------------------------------
//		 ~CSneak3viewApp
// ---------------------------------------------------------------------------
//	Destructor

CSneak3viewApp::~CSneak3viewApp()
{
		// let the tests do any clean up that they need to
	S3Test_Exit();
}


// ===========================================================================
	void
CSneak3viewApp::StartUp()
{
	FSSpec prefsFSSpec;
	
	try	{	
		if (CSettingsDoc::GetPrefsFSSpec(prefsFSSpec)) {
				// we found an existing prefs file, so open it rather than 
				// a blank window
			new CSettingsDoc(this, &prefsFSSpec);
		} else {
				// create a new settings window so the user has something to work with
				// as soon as the app opens
			ObeyCommand(cmd_New, nil);
		}
	}
	
		// we have an empty catch here so that we catch any exceptions thrown 
		// above and then continue on to show the test windows
	catch(...) {
		;
	}
	
		// now show the test windows, after they've been moved and resized
		// by the settings window
	ObeyCommand(cmd_ShowTestWindows, nil);
}


// ===========================================================================
	void
CSneak3viewApp::HandleAppleEvent(
	const AppleEvent	&inAppleEvent,
	AppleEvent			&outAEReply,
	AEDesc				&outResult,
	long				inAENumber)
{
	switch (inAENumber) {
		case ae_OpenDoc:
		case ae_PrintDoc:
				// let LDocApplication do whatever it needs to do to 
				// handle a document being opened or printed from the Finder.
				// we check for these events only because we have to make
				// sure that the test windows get shown after all the 
				// documents have been opened by calling ShowTestWindows().
			LDocApplication::HandleAppleEvent(inAppleEvent, outAEReply,
												outResult, inAENumber);
			ShowTestWindows();
			break;
			
		default:
			LDocApplication::HandleAppleEvent(inAppleEvent, outAEReply,
												outResult, inAENumber);
			break;
	}
}


// ---------------------------------------------------------------------------
//		 ObeyCommand
// ---------------------------------------------------------------------------
//	Respond to commands

Boolean
CSneak3viewApp::ObeyCommand(
	CommandT	inCommand,
	void		*ioParam)
{
	Boolean	cmdHandled = true;
	
	switch (inCommand) {
		case cmd_CreateTestWindows:
			CreateTestWindows();
			break;

		case cmd_CloseTestWindows:
			CloseTestWindows();
			break;

		case cmd_SelectTestWindows:
			SelectTestWindows();
			break;

		case cmd_ShowTestWindows:
			ShowTestWindows();
			break;

		case cmd_HideTestWindows:
			HideTestWindows();
			break;

		case cmd_SaveTestOutput:
			SaveTestOutput();
			break;

		case cmd_RunTest:
			CSettingsDoc::GetFrontMost()->RunTest();
			break;

		case cmd_CompareWindows:
			CompareWindows();
			break;

		case cmd_Quit: {
					// we hide the test and comparison windows before attempting
					// to quit so that when unsaved settings windows are closed
					// and the dialog boxes asking to quit without saving 
					// appear, they will open above the documents, not over the
					// test and comparison windows.
				HideTestWindows();
				CComparisonWindow* comparisonWindow = CComparisonWindow::GetWindow();
				if (comparisonWindow != nil) {
					comparisonWindow->Hide();
				}
				
				cmdHandled = inherited::ObeyCommand(inCommand, ioParam);
	
				if (mState != programState_Quitting) {
						// the quit was canceled, so bring back the test and
						// comparison windows
					ShowTestWindows();
					if (comparisonWindow != nil) {
						comparisonWindow->Show();
					}
				}
			}
			break;

		default:
			cmdHandled = inherited::ObeyCommand(inCommand, ioParam);
			break;
	}
	
	return cmdHandled;
}


// ---------------------------------------------------------------------------
//		 FindCommandStatus
// ---------------------------------------------------------------------------
//	Pass back status of a (menu) command

void
CSneak3viewApp::FindCommandStatus(
	CommandT	inCommand,
	Boolean		&outEnabled,
	Boolean		&outUsesMark,
	Char16		&outMark,
	Str255		outName)
{
	outUsesMark = false;
	
	switch (inCommand) {
   	  case cmd_RunTest:
			outEnabled = (CSettingsDoc::GetFrontMost() != nil);
			break;
			
   	  case cmd_CompareWindows:
			outEnabled = true;
			break;
			
		default:
			LDocApplication::FindCommandStatus(inCommand, outEnabled, outUsesMark,
								outMark, outName);
			break;
	}
}


// ---------------------------------------------------------------------------
//		 OpenDocument
// ---------------------------------------------------------------------------
//	Open a Document file

void
CSneak3viewApp::OpenDocument(
	FSSpec	*inMacFSSpec)
{
	CSettingsDoc *theDoc = new CSettingsDoc(this, inMacFSSpec);
}


// ---------------------------------------------------------------------------
//		 MakeNewDocument
// ---------------------------------------------------------------------------
//	Create a new Document

LModelObject*
CSneak3viewApp::MakeNewDocument()
{
	CSettingsDoc *theDoc = new CSettingsDoc(this, nil);

	return theDoc;
}


// ---------------------------------------------------------------------------
//		 ChooseDocument
// ---------------------------------------------------------------------------
//	Prompt the user to select a document to open

void
CSneak3viewApp::ChooseDocument()
{
	StandardFileReply	macFileReply;
	SFTypeList			typeList;
	
	UDesktop::Deactivate();
	
		// only list files that we've created
	typeList[0] = CSettingsDoc::fileType;

	::StandardGetFile(nil, 1, typeList, &macFileReply);

	UDesktop::Activate();
	
	if (macFileReply.sfGood) {
		OpenDocument(&macFileReply.sfFile);
	}
}


// ===========================================================================
	void
CSneak3viewApp::CreateTestWindows()
{
	if (mLeftTestWindow == nil) {
		mLeftTestWindow = CTinselTestWindow::CreateTinselTestWindow(WIND_TinselTest_Left, this);
	}

	if (mRightTestWindow == nil) {
		mRightTestWindow = CTinselTestWindow::CreateTinselTestWindow(WIND_TinselTest_Right, this);
	}
}


// ===========================================================================
	void
CSneak3viewApp::CloseTestWindows()
{
	delete mLeftTestWindow;
	mLeftTestWindow = nil;

	delete mRightTestWindow;
	mRightTestWindow = nil;
}


// ===========================================================================
	void
CSneak3viewApp::SelectTestWindows()
{
		// we use SelectDeskWindow rather than LWindow::Select because 
		// the former doesn't get recorded by AppleScript, and we don't
		// need this to be recorded
	if (mLeftTestWindow != nil) {
		UDesktop::SelectDeskWindow(mLeftTestWindow);

		GetLeftTinselTestWindow()->Clear();
	}

	if (mRightTestWindow != nil) {
		UDesktop::SelectDeskWindow(mRightTestWindow);
		
		GetRightTinselTestWindow()->Clear();
	}
}


// ===========================================================================
	void
CSneak3viewApp::ShowTestWindows()
{
		// ShowTestWindows should only get called once, after the app is 
		// launched.  we've already opened the splash screen, so now we must
		// close it.  even if ShowTestWindows is called more than once, CAboutBox
		// ignores multiple calls to HideSplashScreen.
	CAboutBox::HideSplashScreen();
	
	if (mLeftTestWindow != nil) {
		mLeftTestWindow->Show();
	}

	if (mRightTestWindow != nil) {
		mRightTestWindow->Show();
	}
}


// ===========================================================================
	void
CSneak3viewApp::HideTestWindows()
{
	if (mLeftTestWindow != nil) {
		mLeftTestWindow->Hide();
	}

	if (mRightTestWindow != nil) {
		mRightTestWindow->Hide();
	}
}


// ===========================================================================
	void
CSneak3viewApp::SaveTestOutput()
{
	if (mLeftTestWindow != nil) {
		mLeftTestWindow->SaveTestOutput();
	}

	if (mRightTestWindow != nil) {
		mRightTestWindow->SaveTestOutput();
	}
}


// ===========================================================================
	void
CSneak3viewApp::CompareWindows()
{
	if (mLeftTestWindow != nil && mRightTestWindow != nil) {
			// the two test windows are open, so call CreateComparisonWindow
			// to make the comparison window.  if one is already open, it
			// will be used.  then compare the test windows.
		CComparisonWindow::CreateComparisonWindow(WIND_Comparison, this);
		CComparisonWindow::CompareWindows(mLeftTestWindow, mRightTestWindow);
	}
}


// ===========================================================================
	void
CSneak3viewApp::ShowAboutBox()
{
	CAboutBox::ShowAboutBox(this);
}


