// ===========================================================================
//	CTexturePopupMenu.cp
// ===========================================================================


#include <Processes.h>
#include <Files.h>

#include "CTexturePopupMenu.h"
#include "StringConversion.h"
#include "Search.h"


MenuHandle CTexturePopupMenu::sMenuH = nil;
ResIDT CTexturePopupMenu::sMenuID = 0;
LList CTexturePopupMenu::sPopupMenuList;
LList CTexturePopupMenu::sDataList(sizeof(FSSpec));
Boolean CTexturePopupMenu::sDoUpdate = true;


// ===========================================================================
	void
CTexturePopupMenu::InitializeMenu()
{
		// find out our process number
	OSErr err;
	ProcessSerialNumber processNum;
	err = GetCurrentProcess(&processNum);
	Assert_(err == noErr);
	
		// set up ProcessInfoRec to get the app's FSSpec
	ProcessInfoRec processInfo;
	processInfo.processInfoLength = sizeof(ProcessInfoRec);
	processInfo.processName = nil;
	FSSpec appFileSpec;
	processInfo.processAppSpec = &appFileSpec;
	
		// get our app's ProcessInfoRec so we can find out where 
		// the app was launched from
	err = GetProcessInformation(&processNum, &processInfo);
	Assert_(err == noErr);
	
	CSParam paramBlock;
	CInfoPBRec searchInfo1, searchInfo2;

/*
		// we don't care what names the files have
	searchInfo1.ioNamePtr = nil;
	searchInfo2.ioNamePtr = nil;

		// we want files only, so clear bit 4 (directory vs. file) 
		// and set bit 4 in the mask
	searchInfo1.ioFlAttrib = $00;
	searchInfo2.ioFlAttrib = $10;

		// we want to search on the fdType, but none of the other fields,
		// so set the FInfo mask to 1 (the first field)
	searchInfo1.ioFlFndrInfo.fdType = 'PICT';
//	searchInfo1.ioFlFndrInfo.fdCreator = 00;
//	searchInfo1.ioFlFndrInfo.fdFlags = 00;
//	searchInfo1.ioFlFndrInfo.fdLocation = 00;
//	searchInfo1.ioFlFndrInfo.fdFldr = 00;
	searchInfo2.ioFlFndrInfo.fdType = 1;
*/

	paramBlock.ioCompletion					= nil;
	paramBlock.ioNamePtr					= nil;
	paramBlock.ioVRefNum					= appFileSpec.vRefNum;

	FSSpec* textureFileSpecs = new FSSpec[kMaxTextureCount];
	paramBlock.ioMatchPtr					= textureFileSpecs;

	paramBlock.ioReqMatchCount					= kMaxTextureCount;

		// search according to the attributes in the 
		// ioFlFndrInfo and ioFlAttrib fields
	paramBlock.ioSearchBits					= fsSBFlFndrInfo + fsSBFlAttrib;
	
	paramBlock.ioSearchInfo1					= &searchInfo1;
	paramBlock.ioSearchInfo2					= &searchInfo2;
	paramBlock.ioSearchTime					= 0;
	paramBlock.ioCatPosition.initialize = 0;
	paramBlock.ioOptBuffer 				= nil;
//	paramBlock.ioOptBuffer 				= @myBuffer;
	paramBlock.ioOptBufSize				 = 0;
//	paramBlock.ioOptBufSize				 = kOptBufferSize;

// causes link error
//	err = IndexedSearch(&paramBlock, appFileSpec.parID);
	
	if ((err == noErr || err == eofErr) && (paramBlock.ioActMatchCount > 0)) {
		CTexturePopupMenu::BeginUpdate();
		
		for (short i = 0; i < paramBlock.ioActMatchCount; i++) {
			CTexturePopupMenu::AppendSharedItem(textureFileSpecs[i].name);
//			CTexturePopupMenu::AppendSharedItem(textureFileSpecs[i].name, &(textureFileSpecs[i]));
		} // endfor
		
		CTexturePopupMenu::EndUpdate();
	}
	
	delete[] textureFileSpecs;
}


// ===========================================================================
	void
CTexturePopupMenu::ReinitializeMenu()
{
	// first get rid of old data values
	// then call InitializeMenu()
	InitializeMenu();
}

/*
CONST
	kMaxMatches 						=	30;			{find up to 30 matches in one pass}
	kOptBufferSize						=	$4000;			{use a 16K search cache for speed}
VAR
	myErr:				OSErr;						{result code of function calls}
	myCount:				Integer;						{loop control variable}
	myFName:				Str255; 						{name of string to look for}
	myVRefNum:				Integer;						{volume to search}
	myDirID:				LongInt;						{ignored directory ID for HGetVol}
	myCurrDate:				LongInt;						{current date, in seconds}
	twoDaysAgo:				LongInt;						{date two days ago, in seconds}
	myPB:				HParamBlockRec;						{parameter block for PBCatSearch}
	myMatches:				PACKED ARRAY[1..kMaxMatches] OF FSSpec;
											{put matches here}
	mySpec1:				CInfoPBRec; 						{search criteria, part 1}
	mySpec2:				CInfoPBRec;					 	{search criteria, part 2}
	myBuffer: 				PACKED ARRAY[1..kOptBufferSize] OF Char;
											{search cache}
	done:				Boolean;						{have all matches been found?}
PROCEDURE SetupForFirstTime;
BEGIN
	myErr := HGetVol(NIL, myVRefNum, myDirID);
														{search on the default volume}
	myFName := 'Temp'; 													{search for "Temp"}
	GetDateTime(myCurrDate);													{get current time in seconds}
	twoDaysAgo := myCurrDate - (2 * 24 * 60 * 60);
	WITH myPB DO
	BEGIN
		ioCompletion					:= NIL;							{no completion routine}
		ioNamePtr					:= NIL;							{no volume name; use vRefNum}
		ioVRefNum					:= myVRefNum;							{volume to search}
		ioMatchPtr					:= FSSpecArrayPtr(@myMatches);
														{points to results buffer}
		ioReqMatchCount					:= kMaxMatches;							{number of matches}
		ioSearchBits					:= fsSBPartialName							{search on partial name}
								+ fsSBFlAttrib						{search on file attributes}
								+ fsSBFlCrDat;						{search on creation date}
		ioSearchInfo1					:= @mySpec1;							{points to first criteria set}
		ioSearchInfo2					:= @mySpec2;							{points to second criteria set}
		ioSearchTime					:= 0;							{no timeout on searches}
		ioCatPosition.initialize := 0;												{set hint to 0}
		ioOptBuffer 				:= @myBuffer; 								{point to search cache}
		ioOptBufSize				 := kOptBufferSize;								{size of search cache}
	END;
	WITH mySpec1 DO
	BEGIN
		ioNamePtr := @myFName; 												{point to string to find}
		ioFlAttrib := $00;												{clear bit 4 to ask for files}
		ioFlCrDat := twoDaysAgo;												{lower bound of creation date}
	END;
	WITH mySpec2 DO
	BEGIN
		ioNamePtr := NIL;												{set to NIL}
		ioFlAttrib := $10;												{set mask for bit 4}
		ioFlCrDat := myCurrDate;												{upper bound of creation date}
	END;
END;
BEGIN
	SetupForFirstTime;													{initialize data records}
	REPEAT
		myErr := PBCatSearchSync(@myPB);												{get some files}
		done := (myErr = eofErr); 												{eofErr returned when all done}
		IF ((myErr = noErr) | done) & (myPB.ioActMatchCount > 0) THEN
			FOR myCount := 1 TO myPB.ioActMatchCount DO
				Writeln(myMatches[myCount].name);
														{report all matches found}
	UNTIL done;
END;
*/


// ===========================================================================
	void
CTexturePopupMenu::UpdatePopupMenus()
{
	if (!sDoUpdate) {
			// we're in the middle of a BeginUpdate/EndUpdate pair, so we don't
			// do the update until EndUpdate is called
		return;
	}
	
	LListIterator iterator(sPopupMenuList, iterate_FromStart);
	CTexturePopupMenu* thePopup = nil;

		// update the control values of each open popup
	while (iterator.Next(&thePopup)) {
		Assert_(thePopup);
		
		thePopup->UpdateValues();
	}
}


// ===========================================================================
	void
CTexturePopupMenu::BeginUpdate()
{
		// we set this flag to false so that UpdatePopupMenus() doesn't update
		// every popup after each call to AppendSharedItem, InsertSharedItem, etc.
		// when EndUpdate is called, this flag will be reset.
	sDoUpdate = false;
}


// ===========================================================================
	void
CTexturePopupMenu::EndUpdate()
{
		// turn updating back on
	sDoUpdate = true;
	
		// make sure all the popups get updated correctly, now that the caller
		// is done changing the shared menu.
	UpdatePopupMenus();
}


// ===========================================================================
// only one item can be appended at a time
//
	void
CTexturePopupMenu::AppendSharedItem(const Str255 inItemString, void* inData)
{
	::AppendMenu(sMenuH, inItemString);
	
		// append the data for this item 
	sDataList.InsertItemsAt(1, arrayIndex_Last, inData);
	
	UpdatePopupMenus();
}


// ===========================================================================
// only one item can be inserted at a time
//
	void
CTexturePopupMenu::InsertSharedItem(const Str255 inItemString, Int16 inAfterItem, void* inData)
{
	::InsertMenuItem(sMenuH, inItemString, inAfterItem);
	
		// insert the data for this item.  inAfterItem is the index of the item
		// after which the new item is inserted, so add 1 to it get the LList index.
	sDataList.InsertItemsAt(1, inAfterItem + 1, inData);
	
	UpdatePopupMenus();
}


//// ===========================================================================
//	void
//CTexturePopupMenu::AppendSharedResourceItems(ResType inResType, void* inData)
//{
//	::AppendResMenu(GetMacMenuH(), inResType);
//	
//	UpdatePopupMenus();
//}
//
//
//// ===========================================================================
//	void
//CTexturePopupMenu::InsertSharedResourceItems(ResType inResType, Int16 inAfterItem)
//{
//	::InsertResMenu(GetMacMenuH(), inResType, inAfterItem);
//	
//	UpdatePopupMenus();
//}


// ===========================================================================
	void
CTexturePopupMenu::DeleteSharedItem(Int16 inItemNumber)
{
	::DeleteMenuItem(sMenuH, inItemNumber);
	
		// remove the item's corresponding data 
	sDataList.RemoveItemsAt(1, inItemNumber);
	
	UpdatePopupMenus();
}


// ===========================================================================
	Int16
CTexturePopupMenu::GetSharedItemFromString(const Str255 inString) const
{
		// Search for inString in the items of this Menu
	Str255 itemString;
	short numItems = ::CountMItems(sMenuH);
	
	for (short theIndex = 1; theIndex <= numItems; theIndex++) {
		::GetMenuItemText(sMenuH, theIndex, itemString);
		
		if (::EqualString(inString, itemString, true, false)) {
			return theIndex;
		}
	}
	
		// the string wasn't found in this menu
	return 0;
}


// ===========================================================================
	Int16
CTexturePopupMenu::GetSharedItemFromString(const char* inString) const
{
	Str255 PString;
	S3CtoPString(inString, PString);
	
			// Search for inString in the items of this Menu
	Str255 itemString;
	short numItems = ::CountMItems(sMenuH);
	
	for (short theIndex = 1; theIndex <= numItems; theIndex++) {
		::GetMenuItemText(sMenuH, theIndex, itemString);
		
		if (::EqualString(PString, itemString, true, false)) {
			return theIndex;
		}
	}
	
		// the string wasn't found in this menu
	return 0;
}


// ===========================================================================
	void*
CTexturePopupMenu::GetSharedDataFromString(const Str255 inString)
{
	Int16 index = GetSharedItemFromString(inString);
	
		// if the index is out of bounds, GetSharedDataFromItem() returns nil
	return GetSharedDataFromItem(index);
}


// ===========================================================================
	void*
CTexturePopupMenu::GetSharedDataFromString(const char* inString)
{
	Int16 index = GetSharedItemFromString(inString);
	
		// if the index is out of bounds, GetSharedDataFromItem() returns nil
	return GetSharedDataFromItem(index);
}


// ===========================================================================
	void*
CTexturePopupMenu::GetSharedDataFromItem(Int16 inIndex) const
{
	void* data = nil;
	
		// if the index is out of bounds, data will remain nil
	sDataList.FetchItemAt(inIndex, &data);
	
	return data;
}


// ===========================================================================
	Int16
CTexturePopupMenu::GetSharedItemFromData(const void* inData) const
{
		// if the index is out of bounds, data will remain nil
	Int32 index = sDataList.FetchIndexOf(inData);
	
	if (index < 1) {
			// the data wasn't found in the list
		return 0;
	}
	
	return (Int16) index;
}


// ===========================================================================
//  CTexturePopupMenu											   CTexturePopupMenu 
// ===========================================================================
	CTexturePopupMenu*
CTexturePopupMenu::CreateTexturePopupMenuStream(LStream	*inStream)
{
	return (new CTexturePopupMenu(inStream));
}


// ===========================================================================
//		 CTexturePopupMenu
// ===========================================================================
//	Construct from input parameters

CTexturePopupMenu::CTexturePopupMenu(
	const SPaneInfo	&inPaneInfo,
	MessageT		inValueMessage,
	Int16			inTitleOptions,
	ResIDT			inMENUid,
	Int16			inTitleWidth,
	Int16			inPopupVariation,
	ResIDT			inTextTraitsID,
	Str255			inTitle,
	OSType			inResTypeMENU,
	Int16			inInitialMenuItem)
		:	CSmartStdPopupMenu(inPaneInfo, inValueMessage, inTitleOptions,
				inMENUid, inTitleWidth, inPopupVariation, inTextTraitsID,
				inTitle, inResTypeMENU, inInitialMenuItem)
{
	InitTexturePopupMenu();
}


// ===========================================================================
//		 CTexturePopupMenu
// ===========================================================================
//	Construct from input parameters and an existing ControlHandle

CTexturePopupMenu::CTexturePopupMenu(
	const SPaneInfo	&inPaneInfo,
	MessageT		inValueMessage,
	Int32			inMaxValue,
	ResIDT			inTextTraitsID,
	ControlHandle	inMacControlH)
		: CSmartStdPopupMenu(inPaneInfo, inValueMessage, inMaxValue,
						inTextTraitsID, inMacControlH)
{
	InitTexturePopupMenu();
}


// ===========================================================================
//		 CTexturePopupMenu(LStream*)
// ===========================================================================
//	Construct from data in a Stream

CTexturePopupMenu::CTexturePopupMenu(
	LStream	*inStream)
		:	CSmartStdPopupMenu(inStream)
{	
	InitTexturePopupMenu();
}


// ===========================================================================
//		 ~CTexturePopupMenu
// ===========================================================================
CTexturePopupMenu::~CTexturePopupMenu()
{	
		// remove us from the list of popup menus
	sPopupMenuList.Remove(&this);
	
		// get a handle to the popup's control record
	ControlHandle controlH = GetMacControl();
	
		// get a handle to the popup's private data record in the control record
	PopupPrivateDataHandle	dataH =
		(PopupPrivateDataHandle) (**controlH).contrlData;

//MenuHandle menuH = (**dataH).mHandle;
//MenuPtr menuP = *menuH;
//char handleState = ::HGetState((Handle) (**dataH).mHandle);
		
		// make the menu handle nil, so that 
	(**dataH).mHandle = nil;
	(**dataH).mID = 9999;
	
		// dispose of the data record so that DisposeControl can't get ahold
		// of it and dispose of the menu handle it contains
//	::DisposeHandle((Handle) dataH);
	
		// set the control's refCon to 0, so that DisposeControl doesn't see
		// anything there to dispose of
//	::SetControlReference(controlH, 0);
}


// ===========================================================================
//		 InitTexturePopupMenu
// ===========================================================================
//	Private initializer

	void
CTexturePopupMenu::InitTexturePopupMenu()
{	
	if (sMenuH == nil) {
			// this is the first popup instance created, so we have to set the 
			// sMenuH class variable to the menu handle created for us by PowerPlant
			// so that all subsequent instances share the same menu
//		sMenuH = inherited::GetMacMenuH();
//		sMenuID = (**sMenuH).menuID;
		sMenuID = (**(inherited::GetMacMenuH())).menuID;
// this should get the handle from the menu list, which may not get released 
// when the control is destroyed
		sMenuH = ::GetMenuHandle(sMenuID);
		
			// we also have to initialize the menu data to hold the texture file specs
		InitializeMenu();
	}
	
		// get a handle to the popup's private data record in the control record
	PopupPrivateDataHandle	dataH =
		(PopupPrivateDataHandle) (**GetMacControl()).contrlData;

		// make the menu handle of this popup point to the static menu shared
		// by all instances of this class
	(**dataH).mHandle = sMenuH;

		// add us to the list of existing popup menus
	sPopupMenuList.InsertItemsAt(1, arrayIndex_Last, &this);
	
	 // save the string of the item currently selected in the popup
	GetStringFromItem(GetValue(), mValueString);

	Int32 itemCount = GetMaxValue();
	Int32 dataCount = sDataList.GetCount();
	
		// either of these two branches should be true only if this is the
		// first popup menu to be created.  after that, the insertion and
		// deletion methods should keep the two lists in sync.
	if (itemCount > dataCount) {
			// the menu has more items than we have data values for, so append
			// nil pointers to the end of sDataList, one for each item that has
			// no data
		sDataList.InsertItemsAt(itemCount - dataCount,
													  arrayIndex_Last, nil);
	} else if (itemCount < dataCount) {
			// the menu has fewer items than we have data values for, so remove
			// the extra data values from the end of sDataList
		sDataList.RemoveItemsAt(dataCount - itemCount, itemCount + 1);
	}
	
	SetBroadcastString(true);
}


// ===========================================================================
	void
CTexturePopupMenu::SetValue(Int32 inValue)
{
	inherited::SetValue(inValue);
	
		// save the string of the item we just set
	GetStringFromItem(GetValue(), mValueString);
}


// ===========================================================================
	void
CTexturePopupMenu::SetValueByData(const void* inData)
{	
	Int16 index = GetSharedItemFromData(inData);
	
	if (index == 0) {
			// this string wasn't found in the menu, so default to the first item
		index = 1;
	}
	
	SetValue(index);
}


// ===========================================================================
	void*
CTexturePopupMenu::GetData() const
{
	return GetSharedDataFromItem(GetValue());
}


// ===========================================================================
	void
CTexturePopupMenu::AppendItem(const Str255 inItemString)
{
	AppendSharedItem(inItemString);
}


// ===========================================================================
	void
CTexturePopupMenu::InsertItem(const Str255 inItemString, Int16 inAfterItem)
{
	InsertSharedItem(inItemString, inAfterItem);
}


//// ===========================================================================
//	void
//CTexturePopupMenu::AppendResourceItems(ResType inResType)
//{
//	AppendSharedResourceItems(inResType);
//}
//
//
//// ===========================================================================
//	void
//CTexturePopupMenu::InsertResourceItems(ResType inResType, Int16 inAfterItem)
//{
//	InsertSharedResourceItems(inResType, inAfterItem);
//}


// ===========================================================================
	void
CTexturePopupMenu::DeleteItem(Int16 inItemNumber)
{
	DeleteSharedItem(inItemNumber);
}


// ===========================================================================
	void
CTexturePopupMenu::UpdateValues()
{
		// determine the maximum value by counting the number of items in 
		// the menu resource
	SetMaxValue(::CountMItems(GetMacMenuH()));
	
	if (GetMaxValue() > 0) {
			// there's at least one item in the menu, so make sure the min
			// value is 1, since we might have just added the first item in the
			// menu, and the min value might still be 0
		SetMinValue(1);
		
			// if items in the menu were changed, the text of the item with 
			// the same number as the value of the popup might be different 
			// than what it was before, because the order of the items has
			// shifted, or the previously selected item was deleted.  although
			// the popup's value is still the same, to the user it won't look 
			// like the same item is selected since a different string appears
			// next to the title.  so reset the value according to the string
			// of the item that was previously selected.  if the item has been
			// deleted (and thus its string can't be found in the menu), 
			// SetValueByString will default to item 1.
		SetValueByString(mValueString);
	} else {
			// there are no items in the menu, so set the other values to
			// reflect that
		SetMinValue(0);
		SetValue(0);
	}
}

