// ===========================================================================
// LTabViewAlternator.cp
// ===========================================================================
//
// assumes the tabs are the only subviews


#include <LCommander.h>
#include <LView.h>

#include "LTabViewAlternator.h"
#include "LCommanderView.h"


// ===========================================================================
	LTabViewAlternator*
LTabViewAlternator::CreateTabViewAlternatorStream(
	LStream	*inStream)
{
	return (new LTabViewAlternator(inStream));
}


// ===========================================================================
LTabViewAlternator::LTabViewAlternator()
{
	// unused default constructor
}


// ===========================================================================
LTabViewAlternator::LTabViewAlternator(LStream* inStream)
	: LViewAlternator(inStream),
		LListener()
{
		// read the location of the sub view from the stream
	inStream->ReadData(&mViewLeft, sizeof(Int16));
	inStream->ReadData(&mViewTop, sizeof(Int16));
	inStream->ReadData(&mViewWidth, sizeof(Int16));
	inStream->ReadData(&mViewHeight, sizeof(Int16));

	inStream->ReadData(&mTabsLeft, sizeof(Int16));
	inStream->ReadData(&mTabsTop, sizeof(Int16));
	inStream->ReadData(&mTabsWidth, sizeof(Int16));
	inStream->ReadData(&mTabsHeight, sizeof(Int16));

	inStream->ReadData(&mTabOverlap, sizeof(Int16));
	
	mCurrentViewIndex = 0;
	mCurrentTabLeft = mTabsLeft;
}


// ===========================================================================
LTabViewAlternator::~LTabViewAlternator()
{
}


// ===========================================================================
	void
LTabViewAlternator::FinishCreateSelf()
{
	LCommander* defaultCommander = LCommander::GetDefaultCommander();
	
	LListIterator tabs(mTabList, iterate_FromStart);

	LTab* theTab;
	
	while (tabs.Next(&theTab)) {
			// get the pane ID of the view which this tab corresponds to
		PaneIDT tabViewID = theTab->GetTabViewID();
		
			// all the subviews will have the same default commander
		LCommander::SetDefaultCommander(defaultCommander);
		
		LCommanderView* theView = CreateView(tabViewID);
		
			// move the view to its new location 
		theView->PlaceInSuperFrameAt(mViewLeft, mViewTop, false);
		
			// store a pointer to the view so we can access it later
		mViewList.InsertItemsAt(1, arrayIndex_Last, &theView);
	}
}


// ===========================================================================
	void
LTabViewAlternator::ListenToMessage(
	MessageT	inMessage,
	void*		ioParam)
{
	switch (inMessage) {
		case msg_ControlClicked: {
			LTab* theTab = (LTab*) ioParam;
			Assert_(theTab);
			
			Int32 newViewIndex = theTab->GetTabIndex();
			
			if (newViewIndex != mCurrentViewIndex) {
					// a new tab has been clicked, so show its corresponding view
				ShowView(newViewIndex);
			}
		}
		break;
	}
}


// ===========================================================================
	void
LTabViewAlternator::DrawSelf()
{
	::PenNormal();
	ApplyForeAndBackColors();
	
	Rect	frame;
	CalcLocalFrameRect(frame);

		// change the frame to be the area in which the subviews are inserted
	frame.left += mViewLeft;
	frame.top += mViewTop;
	frame.right = frame.left + mViewWidth;
	frame.bottom = frame.top + mViewHeight;

	::FrameRect(&frame);
}


// ===========================================================================
	void
LTabViewAlternator::DrawTabs()
{
	Int16 layer = 1;
	
	LListIterator tabs(mTabList, iterate_FromStart);
	LTab* theTab;
	
	while (tabs.Next(&theTab)) {
		theTab->Draw(mUpdateRgnH);
	}
	
	return;
	
	while (tabs.Next(&theTab)) {
		if (theTab->GetDrawLayer() != layer) {
				// we've gone as far as we can in this direction
			break;
		} else {
			theTab->Draw(mUpdateRgnH);
			layer++;
		}
	} // endwhile
	
	tabs.ResetTo(iterate_FromEnd);
	
	while (tabs.Previous(&theTab)) {
		if (theTab->GetDrawLayer() != layer) {
				// we've gone as far as we can in this direction
			break;
		} else {
			theTab->Draw(mUpdateRgnH);
			layer++;
		}
	} // endwhile
}


// ===========================================================================
	void
LTabViewAlternator::DrawCurrentView()
{
	LCommanderView* currentView;
	
	if (mViewList.FetchItemAt(mCurrentViewIndex, &currentView)) {
			// we have a current view, so hide it
		currentView->Draw(mUpdateRgnH);
	}
}


// ===========================================================================
	void
LTabViewAlternator::BringTabToFront()
{
	LTab* theTab;
	
	if (mTabList.FetchItemAt(mCurrentViewIndex, &theTab)) {
		Assert_(theTab);
		
		Int16 layer = mTabList.GetCount();
		
		theTab->SetDrawLayer(layer--);
		
		RgnHandle coveredRegion = ::NewRgn();
		ThrowIfNil_(coveredRegion);
		
		::CopyRgn(theTab->GetFullRegion(), coveredRegion);
		
		theTab->SetLayeredRegion(coveredRegion);

		theTab->Refresh();
		
		LListIterator tabs(mTabList, mCurrentViewIndex);
		
		RgnHandle uncoveredRegion = ::NewRgn();
		ThrowIfNil_(uncoveredRegion);
		
		while (tabs.Next(&theTab)) {
			Assert_(theTab);
			
			theTab->SetDrawLayer(layer--);
			
				// set the tab's region to be the full tab minus the area
				// of the tabs which cover it
			::DiffRgn(theTab->GetFullRegion(), coveredRegion, uncoveredRegion);
			theTab->SetLayeredRegion(uncoveredRegion);
			
				// the region which will be covered for the next tab is the union
				// of the currently uncovered region for this tab and the 
				// covered region
			::UnionRgn(coveredRegion, uncoveredRegion, coveredRegion);
			
			theTab->Refresh();
		} // endwhile
		
		tabs.ResetTo(mCurrentViewIndex);
		
		while (tabs.Previous(&theTab)) {
			Assert_(theTab);
			
			theTab->SetDrawLayer(layer--);

				// set the tab's region to be the full tab minus the area
				// of the tabs which cover it
			::DiffRgn(theTab->GetFullRegion(), coveredRegion, uncoveredRegion);
			theTab->SetLayeredRegion(uncoveredRegion);
			
				// the region which will be covered for the next tab is the union
				// of the currently uncovered region for this tab and the 
				// covered region
			::UnionRgn(coveredRegion, uncoveredRegion, coveredRegion);
			
			theTab->Refresh();
		} // endwhile
		
		if (uncoveredRegion != nil) {
			::DisposeRgn(uncoveredRegion);
		}
		
		if (coveredRegion != nil) {
			::DisposeRgn(coveredRegion);
		}
	}
}


// ===========================================================================
	void
LTabViewAlternator::AddTab(LTab* inTab)
{
	Assert_(inTab);
	
		// save a pointer to this tab
	mTabList.InsertItemsAt(1, arrayIndex_Last, &inTab);
	
		// give the tab an index value which is the same as its index in our
		// tab list, and which will be the same as the tab view's index
	inTab->SetTabIndex(mTabList.GetCount());
	
	inTab->SetDrawLayer(mTabList.GetCount());
	
		// we become a listener of this tab so we'll know when it gets clicked
	inTab->AddListener(this);

		// position the tab just to the right of the current tab
	inTab->PlaceInSuperFrameAt(mCurrentTabLeft, mTabsTop, false);

		// get the size of the tab
	SDimension16 tabSize;
	inTab->GetFrameSize(tabSize);

		// the next tab will be placed to the right of the current one
		// overlapping it if mTabOverlap > 0
	mCurrentTabLeft += (tabSize.width - mTabOverlap);
}


// ===========================================================================
	void
LTabViewAlternator::ShowView(Int32 inViewIndex)
{
	LTab* newTab;
	
	if (mTabList.FetchItemAt(inViewIndex, &newTab)) {
			// inViewIndex is a valid index
		LTab* currentTab;
		
		if (mTabList.FetchItemAt(mCurrentViewIndex, &currentTab)) {
				// we have a current tab, so turn it off
			currentTab->SetValue(Button_Off);
		}
		
		newTab->SetValue(Button_On);
	}
		
		// let LViewAlternator handle switching the views
	LViewAlternator::ShowView(inViewIndex);
	
	BringTabToFront();
}


// ===========================================================================
	void
LTabViewAlternator::ShowNextView()
{
	LTab* theTab;
	
	if (mTabList.FetchItemAt(mCurrentViewIndex, &theTab)) {
			// we have a current tab, so turn it off
		theTab->SetValue(Button_Off);
	}

		// let LViewAlternator hide the current view and show the next.  we
		// call it in the middle here so that it will increment mCurrentViewIndex
	LViewAlternator::ShowNextView();
	
	if (mTabList.FetchItemAt(mCurrentViewIndex, &theTab)) {
			// turn on the next tab
		theTab->SetValue(Button_On);
	}
	
	BringTabToFront();
}


// ===========================================================================
	void
LTabViewAlternator::ShowPreviousView()
{
	LTab* theTab;
	
	if (mTabList.FetchItemAt(mCurrentViewIndex, &theTab)) {
			// we have a current tab, so turn it off
		theTab->SetValue(Button_Off);
	}

		// let LViewAlternator hide the current view and show the previous.  we
		// call it in the middle here so that it will decrement mCurrentViewIndex
	LViewAlternator::ShowPreviousView();
	
	if (mTabList.FetchItemAt(mCurrentViewIndex, &theTab)) {
			// turn on the previous tab
		theTab->SetValue(Button_On);
	}
	
	BringTabToFront();
}


