// ===========================================================================
// S3SceneFile.c
// ===========================================================================


#include <stdio.h>
#include <string.h>

#include "S3TestInfo.h"
#include "S3TestSystem.h"
#include "S3SceneFile.h"
#include "S3Utilities.h"
#include "S3Image.h"


// ---------------------------------------------------------------------------
// Globals
// ---------------------------------------------------------------------------

unsigned long				gSceneWidth, gSceneHeight;

float						gXscale, gYscale;

S3Image						gTextureImage = nil;
TQATexture*					gTextureLeft = nil;
TQATexture*					gTextureRight = nil;

S3tTestInfo*				gTestInfoP = nil;


// ---------------------------------------------------------------------------
// Routines
// ---------------------------------------------------------------------------

// ===========================================================================
	Boolean
S3SceneFile_Open(
	FILE**					inSceneFileP,
	char*					inFileName)
{
	*inSceneFileP = fopen(inFileName, "r");

	return (*inSceneFileP != nil);
}


// ===========================================================================
	void
S3SceneFile_Close(
	FILE*					inSceneFile)
{
	fclose(inSceneFile);
}


// ===========================================================================
	Boolean
S3SceneFile_ReadAndDraw(
	FILE*					inSceneFile,
	S3tTestInfo*			inTestInfo)
{
	Boolean					finished = false;
	char					c;
	
	gTestInfoP = inTestInfo;
	
	if (S3SceneFile_SetScene(inSceneFile, inTestInfo) == false) {
			// we got an error while reading in the scene info
		return false;
	}
	
	S3Test_RenderStart(inTestInfo, NULL);
	
	while (!finished) {
		c = fgetc(inSceneFile);
		switch (c) {
			case 'T':
				if (S3SceneFile_DrawTextureTri(inSceneFile, inTestInfo) == false) {
					finished = true;
				}
				break;
				
			case 'G':
				if (S3SceneFile_DrawGouraudTri(inSceneFile, inTestInfo) == false) {
					finished = true;
				}
				break;
			
			default:
				finished = true;
				break;
		} // endswitch
		;
	} // endwhile
	
	S3Test_RenderEnd(inTestInfo, NULL);

	if (gTextureImage != nil) {
		S3Image_Delete(gTextureImage);
		gTextureImage = nil;
	}
	
	if (gTextureLeft != nil) {
		QATextureDelete(S3TestInfo_GetLEngine(gTestInfoP), gTextureLeft);
		gTextureLeft = nil;
	}
	
	if (gTextureRight != nil) {
		QATextureDelete(S3TestInfo_GetREngine(gTestInfoP), gTextureRight);
		gTextureRight = nil;
	}
	
	gTestInfoP = nil;
	
	return true;
}


// ===========================================================================
	Boolean
S3SceneFile_SetScene(
	FILE*					inSceneFile,
	S3tTestInfo*			inTestInfo)
{
	float					width = S3TestInfo_GetWidth(inTestInfo);
	float					height = S3TestInfo_GetHeight(inTestInfo);
	float					a, r, g, b;
	int						scanResult;

		// get the width and height of the original scene
	scanResult = fscanf(inSceneFile, "%lu %lu", &gSceneWidth, &gSceneHeight);
	
	if (scanResult == EOF) return false;
	
	S3Test_Print("Original draw context size: %f X %f\r\r", 
			gSceneWidth, gSceneHeight);
	
		// calculate the scaling factor to fit the original XY values into 
		// the current window size.  we'll multiply every XY coord we get
		// by this factor.
	gXscale = width / (float) gSceneWidth;
	gYscale = height / (float) gSceneHeight;

// also store and set values for perspectiv Z, etc.?
	
		// get the background argb values
	scanResult = fscanf(inSceneFile, "%f %f %f %f", 
			&a, &r, &g, &b);
	
	if (scanResult == EOF) return false;

		// set the two contexts' argb values
	S3Test_SetFloat(inTestInfo, kQATag_ColorBG_a, a);

	S3Test_SetFloat(inTestInfo, kQATag_ColorBG_r, r);

	S3Test_SetFloat(inTestInfo, kQATag_ColorBG_g, g);

	S3Test_SetFloat(inTestInfo, kQATag_ColorBG_b, b);
	
		// skip past the return on the end of the line
	fgetc(inSceneFile);
	
	return true;
}


// ===========================================================================
	Boolean
S3SceneFile_DrawTextureTri(
	FILE*					inSceneFile,
	S3tTestInfo*			inTestInfo)
{
	TQAVTexture				v[3];
//	TQAVGouraud				v[3];
	unsigned long			flags, texture;
	int						scanResult;

		// T flags  texture
	scanResult = fscanf(inSceneFile, "%*s   %lu   %x",
			&flags, &texture);
	
	if (scanResult == EOF) return false;
	
	if (gTextureImage == nil) {
		PicHandle					pictH;
		FSSpec 						textureFSSpec;
		TQAError					qaError;
		
		S3GetAppFileSpec(&textureFSSpec);
		
		sprintf((char*) &textureFSSpec.name[1], "%X", texture);
		textureFSSpec.name[0] = strlen((char*) &textureFSSpec.name[1]);
		
		pictH = S3GetPICTFromFile(&textureFSSpec);
		
		gTextureImage = S3Image_New(pictH, kQAPixel_RGB32, false, true);
		
		KillPicture(pictH);
		
			// try to create a texture for the left engine
		qaError = QATextureNew(
			S3TestInfo_GetLEngine(gTestInfoP),
			kQATexture_Mipmap,
			kQAPixel_RGB32,
			S3Image_GetImages(gTextureImage),
			&gTextureLeft);

			// try to create a texture for the right engine
		qaError = QATextureNew(
			S3TestInfo_GetREngine(gTestInfoP),
			kQATexture_Mipmap,
			kQAPixel_RGB32,
			S3Image_GetImages(gTextureImage),
			&gTextureRight);

			// set the left draw context's texture
		QASetPtr(
			S3TestInfo_GetLDrawContext(gTestInfoP), 
			kQATag_Texture, 
			gTextureLeft);

			// set the right draw context's texture
		QASetPtr(
			S3TestInfo_GetRDrawContext(gTestInfoP), 
			kQATag_Texture, 
			gTextureRight);
	}
	
		// T x y z invW   r g b a   uOverW vOverW   kd_r kd_g kd_b   ks_r ks_g ks_b
	scanResult = fscanf(inSceneFile, "%*s %f %f %f %f   %f %f %f %f   %f %f   %f %f %f   %f %f %f", 
//	scanResult = fscanf(inSceneFile, "%*s %f %f %f %f   %f %f %f %f   %*f %*f   %*f %*f %*f   %*f %*f %*f", 
//	scanResult = fscanf(inSceneFile, kMarker_TextureTri"1 %f %f %f %f   %f %f %f %f   %*f %*f   %*f %*f %*f   %*f %*f %*f", 
			&v[0].x, &v[0].y, &v[0].z, &v[0].invW,
			&v[0].r, &v[0].g, &v[0].b, &v[0].a,
//			&v[0].r, &v[0].g, &v[0].b, &v[0].a);
			&v[0].uOverW, &v[0].vOverW,
			&v[0].kd_r, &v[0].kd_g, &v[0].kd_b,
			&v[0].ks_r, &v[0].ks_g, &v[0].ks_r);
	
	if (scanResult == EOF) return false;
	
	v[0].x *= gXscale;
	v[0].y *= gYscale;
	
		// T x y z invW   r g b a   uOverW vOverW   kd_r kd_g kd_b   ks_r ks_g ks_b
	scanResult = fscanf(inSceneFile, "%*s %f %f %f %f   %f %f %f %f   %f %f   %f %f %f   %f %f %f", 
//	scanResult = fscanf(inSceneFile, "%*s %f %f %f %f   %f %f %f %f   %*f %*f   %*f %*f %*f   %*f %*f %*f", 
//	scanResult = fscanf(inSceneFile, kMarker_TextureTri"2 %f %f %f %f   %f %f %f %f   %*f %*f   %*f %*f %*f   %*f %*f %*f", 
			&v[1].x, &v[1].y, &v[1].z, &v[1].invW,
			&v[1].r, &v[1].g, &v[1].b, &v[1].a,
//			&v[1].r, &v[1].g, &v[1].b, &v[1].a);
			&v[1].uOverW, &v[1].vOverW,
			&v[1].kd_r, &v[1].kd_g, &v[1].kd_b,
			&v[1].ks_r, &v[1].ks_g, &v[1].ks_r);
	
	if (scanResult == EOF) return false;
	
	v[1].x *= gXscale;
	v[1].y *= gYscale;
	
		// T x y z invW   r g b a   uOverW vOverW   kd_r kd_g kd_b   ks_r ks_g ks_b flags
	scanResult = fscanf(inSceneFile, "%*s %f %f %f %f   %f %f %f %f   %f %f   %f %f %f   %f %f %f", 
//	scanResult = fscanf(inSceneFile, "%*s %f %f %f %f   %f %f %f %f   %*f %*f   %*f %*f %*f   %*f %*f %*f", 
//	scanResult = fscanf(inSceneFile, kMarker_TextureTri"3 %f %f %f %f   %f %f %f %f   %*f %*f   %*f %*f %*f   %*f %*f %*f %lu", 
			&v[2].x, &v[2].y, &v[2].z, &v[2].invW,
			&v[2].r, &v[2].g, &v[2].b, &v[2].a,
//			&v[2].x, &v[2].y, &v[2].z, &v[2].invW,
//			&v[2].r, &v[2].g, &v[2].b, &v[2].a);
			&v[2].uOverW, &v[2].vOverW,
			&v[2].kd_r, &v[2].kd_g, &v[2].kd_b,
			&v[2].ks_r, &v[2].ks_g, &v[2].ks_r);
	
	if (scanResult == EOF) return false;
	
	v[2].x *= gXscale;
	v[2].y *= gYscale;
	
//	S3Test_DrawTriGouraud(inTestInfo, &v[0], &v[1], &v[2], flags);
	S3Test_DrawTriTexture(inTestInfo, &v[0], &v[1], &v[2], flags);
	
		// skip past the return on the end of the line
	fgetc(inSceneFile);
	
	return true;
}


// ===========================================================================
	Boolean
S3SceneFile_DrawGouraudTri(
	FILE*					inSceneFile,
	S3tTestInfo*			inTestInfo)
{
	TQAVGouraud				v[3];
	unsigned long			flags;
	int						scanResult;
	
		// G x y z invW   r g b a
	scanResult = fscanf(inSceneFile, "%*s %f %f %f %f   %f %f %f %f",
			&v[0].x, &v[0].y, &v[0].z, &v[0].invW,
			&v[0].r, &v[0].g, &v[0].b, &v[0].a);
	
	if (scanResult == EOF) return false;
	
	v[0].x *= gXscale;
	v[0].y *= gYscale;
	
		// G x y z invW   r g b a
	scanResult = fscanf(inSceneFile, "%*s %f %f %f %f   %f %f %f %f",
			&v[1].x, &v[1].y, &v[1].z, &v[1].invW,
			&v[1].r, &v[1].g, &v[1].b, &v[1].a);
	
	if (scanResult == EOF) return false;
	
	v[1].x *= gXscale;
	v[1].y *= gYscale;
	
		// G x y z invW   r g b a   flags
	scanResult = fscanf(inSceneFile, "%*s %f %f %f %f   %f %f %f %f   %lu", 
			&v[2].x, &v[2].y, &v[2].z, &v[2].invW,
			&v[2].r, &v[2].g, &v[2].b, &v[2].a,
			&flags);
	
	if (scanResult == EOF) return false;
	
	v[2].x *= gXscale;
	v[2].y *= gYscale;
	
	S3Test_DrawTriGouraud(inTestInfo, &v[0], &v[1], &v[2], flags);
	
		// skip past the return on the end of the line
	fgetc(inSceneFile);
	
	return true;
}
