/*****************************************************************************\

	S3TestUtil.c

	Copyright (C) 1995 Apple Computer, Inc.  All rights reserved.

\*****************************************************************************/

/*=========================================================================*\
	Includes
\*=========================================================================*/
#include <stdio.h>
#include <stdlib.h>
//#include <math.h>

#include "RAVE.h"
#include "S3TestUtil.h"
#include "S3TestInfo.h"
#include "S3TestSystem.h"

/*=========================================================================*\
	Constants
\*=========================================================================*/

/*=========================================================================*\
	Types
\*=========================================================================*/

/*=========================================================================*\
	Static Utilities
\*=========================================================================*/

/*=========================================================================*\
	TT Utility Routines
\*=========================================================================*/

/*=========================================================================*\
	S3TestUtil_RandomVGouraud
\*=========================================================================*/
void S3TestUtil_RandomVGouraud (
	S3tTestInfo			*testInfo,
	TQAVGouraud			*v)
{
	v->x = S3TestUtil_RandomFloat() * ((float) S3TestInfo_GetWidth(testInfo) - 0.01);
	v->y = S3TestUtil_RandomFloat() * ((float) S3TestInfo_GetHeight(testInfo) - 0.01);
	v->z = S3TestUtil_RandomFloat();
	v->invW = 1.0 / v->z;
	
	v->a = 1.0;
	v->r = S3TestUtil_RandomFloat();
	v->g = S3TestUtil_RandomFloat();
	v->b = S3TestUtil_RandomFloat();
}

/*=========================================================================*\
	S3TestUtil_RandomVTexture
\*=========================================================================*/
void S3TestUtil_RandomVTexture(
	S3tTestInfo			*testInfo,
	TQAVTexture			*v)
{
	v->x = S3TestUtil_RandomFloat() * ((float) S3TestInfo_GetWidth(testInfo) - 0.01);
	v->y = S3TestUtil_RandomFloat() * ((float) S3TestInfo_GetHeight(testInfo) - 0.01);
	v->z = S3TestUtil_RandomFloat();
	v->invW = 1.0 / v->z;
	
	v->uOverW *= v->invW;
	v->vOverW *= v->invW;
	
	v->a = 1.0;
	v->r = S3TestUtil_RandomFloat();
	v->g = S3TestUtil_RandomFloat();
	v->b = S3TestUtil_RandomFloat();
	
	v->kd_r = 1.0;
	v->kd_g = 1.0;
	v->kd_b = 1.0;
	
	v->ks_r = 0.0;
	v->ks_g = 0.0;
	v->ks_b = 0.0;
}

/*=========================================================================*\
	S3TestUtil_RandomAlphaVGouraud
\*=========================================================================*/
void S3TestUtil_RandomAlphaVGouraud (
	S3tTestInfo			*testInfo,
	TQAVGouraud			*v)
{
	S3TestUtil_RandomVGouraud(testInfo, v);
	
	v->a = S3TestUtil_RandomFloat();
}

/*=========================================================================*\
	S3TestUtil_RandomAlphaVTexture
\*=========================================================================*/
void S3TestUtil_RandomAlphaVTexture (
	S3tTestInfo			*testInfo,
	TQAVTexture			*v)
{
	S3TestUtil_RandomVTexture(testInfo, v);
	
	v->a = S3TestUtil_RandomFloat();
}

/*=========================================================================*\
	S3TestUtil_DrawRandomTriGouraud
\*=========================================================================*/
void S3TestUtil_DrawRandomTriGouraud(
	S3tTestInfo			*testInfo)
{
	TQAVGouraud		v[3];

	S3TestUtil_RandomVGouraud(testInfo, &v[0]);
	S3TestUtil_RandomVGouraud(testInfo, &v[1]);
	S3TestUtil_RandomVGouraud(testInfo, &v[2]);
	
	S3Test_DrawTriGouraud(testInfo, &v[0], &v[1], &v[2], kQATriFlags_None);
}

/*=========================================================================*\
	S3TestUtil_DrawRandomTriTexture
\*=========================================================================*/
void S3TestUtil_DrawRandomTriTexture(
	S3tTestInfo			*testInfo)
{
	TQAVTexture		v[3];

	/*
	 * Setup the u,v values before calling S3TestUtil_RandomVTexture(),
	 * as it references them.
	 */
	
	v[0].uOverW = 0.0;
	v[0].vOverW = 0.0;

	v[1].uOverW = 0.0;
	v[1].vOverW = 1.0;

	v[2].uOverW = 1.0;
	v[2].vOverW = 1.0;

	S3TestUtil_RandomVTexture(testInfo, &v[0]);
	S3TestUtil_RandomVTexture(testInfo, &v[1]);
	S3TestUtil_RandomVTexture(testInfo, &v[2]);
	
	S3Test_DrawTriTexture(testInfo, &v[0], &v[1], &v[2], kQATriFlags_None);
}

/*=========================================================================*\
	S3TestUtil_TQAErrorToString
\*=========================================================================*/
char *S3TestUtil_TQAErrorToString(
	TQAError		error)
{
	typedef struct {
		char		*errorMessage;
		TQAError	error;
	} S3tErrorTable;
	
	static S3tErrorTable	table[] =
	{
		{"kQANoErr"				, kQANoErr			},
		{"kQAError"				, kQAError			},
		{"kQAOutOfMemory"		, kQAOutOfMemory	},
		{"kQANotSupported"		, kQANotSupported	},
		{"kQAOutOfDate"			, kQAOutOfDate		},
		{"kQAParamErr"			, kQAParamErr		},
		{"kQAGestaltUnknown"	, kQAGestaltUnknown	}
	};
	
	long	i;
	long	nErr;
	
	nErr = sizeof(table) / sizeof(S3tErrorTable);
	for (i = 0; i < nErr; i++)
	{
		if (table[i].error == error)
			return table[i].errorMessage;
	}
	
	return "Unknown TQAError Code";
}

/*=========================================================================*\
	S3TestUtil_RandomFloat
\*=========================================================================*/
float S3TestUtil_RandomFloat(void)
{
	return (rand() / (float) RAND_MAX);
}

/*=========================================================================*\
	S3TestUtil_ResetRandomSeed
\*=========================================================================*/
void S3TestUtil_ResetRandomSeed(void)
{
	srand(1);
}


