// ===========================================================================
//	StContinueDialog.cp
// ===========================================================================


// ===========================================================================
//  Includes
// ===========================================================================

#include <LUndoer.h>
#include <LWindow.h>
#include <UReanimator.h>

#include "StContinueDialog.h"
#include "CComparisonWindow.h"
#include "S3Messages.h"


// ===========================================================================
//  Static
// ===========================================================================

Point						StContinueDialog::sLastPosition = { 0, 0 };


// ---------------------------------------------------------------------------
//		 StContinueDialog
// ---------------------------------------------------------------------------
//	Constructor

StContinueDialog::StContinueDialog(
	LCommander*				inSuper)
		: StDialogHandler(continueDialog_ResID, inSuper)
{
	if (sLastPosition.h != 0 || sLastPosition.v != 0) {
			// the continue dialog box has already been opened once, so we
			// want to return it to its last position.  otherwise, it will
			// automatically be positioned on the main screen.
		mDialog->DoSetPosition(sLastPosition);
	}
	
		// the dialog starts out hidden, so now that we've positioned it,
		// we can show it
	mDialog->Show();
}


// ---------------------------------------------------------------------------
//		 ~StContinueDialog
// ---------------------------------------------------------------------------
//	Destructor

StContinueDialog::~StContinueDialog()
{
	Point					windowOrigin = 
			topLeft(mDialog->GetMacPort()->portRect);
			
	mDialog->PortToGlobalPoint(windowOrigin);
			
		// save the position of the dialog so that the next time it's opened,
		// we can show it in the same place
	sLastPosition = windowOrigin;
}


// ---------------------------------------------------------------------------
//		 FindCommandStatus
// ---------------------------------------------------------------------------
//	Pass back the status of a Command

	void
StContinueDialog::FindCommandStatus(
	CommandT	inCommand,
	Boolean		&outEnabled,
	Boolean&	outUsesMark,
	Char16&		outMark,
	Str255		outName)
{
		// Don't enable any commands except cmd_About, which will keep
		// the Apple menu enabled. This function purposely does not
		// call the inherited FindCommandStatus, thereby suppressing
		// commands that are handled by SuperCommanders. Only those
		// commands enabled by SubCommanders will be active.
		//
		// This is usually what you want for a moveable modal dialog.
		// Commands such as "New", "Open" and "Quit" that are handled
		// by the Applcation are disabled, but items within the dialog
		// can enable commands. For example, an EditField would enable
		// items in the "Edit" menu.
		// 
		// however, in this case, in addition to the Apple menu, we want
		// to enable some of the commands in the Comparison menu so that 
		// the user can switch channels in the comparison window.
	switch (inCommand) {
		case cmd_About:
			outEnabled = true;
			break;
		
			// we let the comparison window set the status of only those
			// commands which are available when the window is open.  we
			// don't want the window to in turn call LWindow::FindCommandStatus,
			// because that will keep calling up the commander chain, which
			// will enable most of the commands.
		case cmd_CompareRed:
		case cmd_CompareGreen:
		case cmd_CompareBlue:
		case cmd_CompareAlpha:
		case cmd_CompareLeft:
		case cmd_CompareRight:
		case cmd_CompareAverage:
		case cmd_CompareAlternate:
			if (CComparisonWindow::GetWindow() != nil) {
					// the comparison window is open, so ask it for the mark,
					// enable state, etc. of these commands
				CComparisonWindow::GetWindow()->FindCommandStatus(inCommand,
						outEnabled, outUsesMark, outMark, outName);
						
					// make sure to break now that we've set the mark, etc.
				break;
			} 
			
			// *** Fall Through *** 
			// to the default case if the comparison window isn't open
		
		default:
			outEnabled = false;
			break;
	} // endswitch
}


