/* ======================================================================== *\
	S3TestInfo.h
\* ======================================================================== */

#pragma once

/* ======================================================================== *\
	Includes
\* ======================================================================== */
#include <Types.h>

#include "RAVE.h"

#include "S3Image.h"
#include "S3PICTList.h"

#ifdef __cplusplus
extern "C" {
#endif

/* ======================================================================== *\
	Constants
\* ======================================================================== */

#define 					kMaxArgStringLength 			256
#define 					kMaxEngineNameLength 			256
#define 					kMaxTestNameLength 				256
#define 					kTestResultBufferSize			256

typedef enum {
	kS3Status_Failure,
	kS3Status_Completed,
	kS3Status_Cancelled,
	kS3Status_NoSuchTest,
	kS3Status_PrereqsNotMet
} S3tTestStatus;

typedef enum S3eDevice {
	kS3Device_Window = 1,
	kS3Device_Memory16,
	kS3Device_Memory32
} S3eDevice;

typedef enum S3eColorTableFormat {
	kS3ColorTableFormat_16 = 1,
	kS3ColorTableFormat_32
} S3eColorTableFormat;

typedef enum S3eClipRegion {
	kS3ClipRegion_None = 1,
	kS3ClipRegion_RoundedRectangle,
	kS3ClipRegion_Circle,
	kS3ClipRegion_HorizontalLines,
	kS3ClipRegion_VerticalLines,
	kS3ClipRegion_Checkerboard
} S3eClipRegion;

typedef enum TS3Error {
	kS3Error_None = 0,
	kS3Error_General,
	kS3Error_NotEnoughMemory,
	kS3Error_FileNotFound,
	kS3Error_FileSystem,
	kS3Error_RAVE,
	kS3Error_OS,
	kS3Error_Parameter
} TS3Error;

typedef enum TS3TagLong {
	kS3TagLong_DeviceType = 0,
	kS3TagLong_ClipRegion,
	kS3TagLong_ContextFlags,
	kS3TagLong_ZFunction,
	kS3TagLong_Blending,
	kS3TagLong_Antialiasing,
	kS3TagLong_TextureFormat,
	kS3TagLong_TextureFilter,
	kS3TagLong_TextureOp,
	kS3TagLong_TextureFlags,
	kS3TagLong_TextureDetach,
	kS3TagLong_TextureMakeBlackTransparent,
	kS3TagLong_TextureUseTransparentIndex,
	kS3TagLong_BitmapFormat,
	kS3TagLong_BitmapFlags,
	kS3TagLong_BitmapDetach,
	kS3TagLong_BitmapMakeBlackTransparent,
	kS3TagLong_BitmapUseTransparentIndex
} TS3TagLong;

typedef enum TS3TagFloat {
	kS3TagFloat_Alpha = 0,
	kS3TagFloat_Red,
	kS3TagFloat_Green,
	kS3TagFloat_Blue,
	kS3TagFloat_LineWidth
} TS3TagFloat;

typedef enum TS3TagBlock {
	kS3TagBlock_TestName = 0,
	kS3TagBlock_ArgString,
	kS3TagBlock_TestResult,
	kS3TagBlock_LeftEngineName,
	kS3TagBlock_RightEngineName,
	kS3TagBlock_LeftRect,
	kS3TagBlock_RightRect,
	kS3TagBlock_LeftGlobalRect,
	kS3TagBlock_RightGlobalRect,
	kS3TagBlock_TextureFileSpec,
	kS3TagBlock_BitmapFileSpec
} TS3TagBlock;

typedef enum TS3TagBlock_Size {
	kS3TagBlock_Size_TestName = 			kMaxTestNameLength,
	kS3TagBlock_Size_ArgString = 			kMaxArgStringLength,
	kS3TagBlock_Size_TestResult = 			kMaxArgStringLength,
	kS3TagBlock_Size_LeftEngineName =		kMaxEngineNameLength,
	kS3TagBlock_Size_RightEngineName = 		kTestResultBufferSize,
	kS3TagBlock_Size_LeftRect =				sizeof(TQARect),
	kS3TagBlock_Size_RightRect = 			sizeof(TQARect),
	kS3TagBlock_Size_LeftGlobalRect =		sizeof(Rect),
	kS3TagBlock_Size_RightGlobalRect = 		sizeof(Rect),
	kS3TagBlock_Size_TextureFileSpec =		sizeof(FSSpec),
	kS3TagBlock_Size_BitmapFileSpec = 		sizeof(FSSpec)
} TS3TagBlock_Size;

/* ======================================================================== *\
	Types
\* ======================================================================== */

typedef struct S3tTestInfo	S3tTestInfo;
typedef S3tTestInfo*		S3tTestInfoP;

/* ======================================================================== *\
	Routines
\* ======================================================================== */


	S3tTestStatus
S3TestInfo_RunTest(
	S3tTestInfoP			inTestInfo,
	const char*				inTestName);
	
	TS3Error
S3TestInfo_Refresh(
	S3tTestInfoP			inTestInfo);
	
	Boolean
S3TestInfo_AskToContinue(
	S3tTestInfoP			inTestInfo);

	Boolean
S3TestInfo_CheckForCancel(
	S3tTestInfoP			inTestInfo);
	
	TS3Error
S3TestInfo_SetTextureFromFileSpec(
	S3tTestInfoP			inTestInfo,
	const FSSpec*			inFileSpec);
	
	TS3Error
S3TestInfo_SetTextureFromPICTHandle(
	S3tTestInfoP			inTestInfo,
	PicHandle				inPictH);
	
	TS3Error
S3TestInfo_SetBitmapFromFileSpec(
	S3tTestInfoP			inTestInfo,
	const FSSpec*			inFileSpec);
	
	TS3Error
S3TestInfo_SetBitmapFromPICTHandle(
	S3tTestInfoP			inTestInfo,
	PicHandle				inPictH);

	TQAError
S3TestInfo_GetLastRAVEError(
	S3tTestInfoP			inTestInfo);

	
/*
	Tag access to S3tTestInfo
*/


TS3Error S3TestInfo_GetLong(
	S3tTestInfoP		inTestInfo,
	TS3TagLong			inTag,
	unsigned long*		inLongP);

TS3Error S3TestInfo_SetLong(
	S3tTestInfoP		inTestInfo,
	TS3TagLong			inTag,
	unsigned long		inLong);

TS3Error S3TestInfo_GetFloat(
	S3tTestInfoP		inTestInfo,
	TS3TagFloat			inTag,
	float*				inFloatP);

TS3Error S3TestInfo_SetFloat(
	S3tTestInfoP		inTestInfo,
	TS3TagFloat			inTag,
	float				inFloat);

	TS3Error 
S3TestInfo_GetBlock(
	S3tTestInfoP			inTestInfo,
	TS3TagBlock				inTag,
	void*					outBlockP,
	TS3TagBlock_Size		inBlockSize);
	
	TS3Error 
S3TestInfo_SetBlock(
	S3tTestInfoP			inTestInfo,
	TS3TagBlock				inTag,
	const void*				inBlockP);


/*
	L,R Microseconds
*/
unsigned long S3TestInfo_GetLMicroseconds(
	S3tTestInfoP		testInfo);

unsigned long S3TestInfo_GetRMicroseconds(
	S3tTestInfoP		testInfo);

/*
	Test Name
*/
StringPtr S3TestInfo_GetTestName(
	S3tTestInfoP		testInfo);

/*
	Test Argument String
*/
char* S3TestInfo_GetArgString(
	S3tTestInfoP		testInfo);

/*
	PICT list
*/
S3PICTList S3TestInfo_GetPICTList(
	S3tTestInfoP		testInfo);

/*
	L,R DrawContext
*/
TQADrawContext *S3TestInfo_GetLDrawContext(
	S3tTestInfoP		testInfo);

TQADrawContext *S3TestInfo_GetRDrawContext(
	S3tTestInfoP		testInfo);

unsigned long S3TestInfo_GetContextFlags(
	S3tTestInfoP		testInfo);

/*
	L,R Engine
*/
TQAEngine *S3TestInfo_GetLEngine(
	S3tTestInfoP		testInfo);

TQAEngine *S3TestInfo_GetREngine(
	S3tTestInfoP		testInfo);

/*
	Width/Height (matches)
*/
long S3TestInfo_GetWidth(
	S3tTestInfoP		testInfo);

long S3TestInfo_GetHeight(
	S3tTestInfoP		testInfo);

/*
	L, R Device
*/
TQADevice *S3TestInfo_GetLDevice(
	S3tTestInfoP		testInfo);

TQADevice *S3TestInfo_GetRDevice(
	S3tTestInfoP		testInfo);

/*
	Texture Image
*/
S3Image S3TestInfo_GetTextureImage(
	S3tTestInfoP		testInfo);

/*
	L, R Texture
*/
TQATexture *S3TestInfo_GetLTexture(
	S3tTestInfoP		testInfo);

TQATexture *S3TestInfo_GetRTexture(
	S3tTestInfoP		testInfo);

/*
	Texture Size
*/
long S3TestInfo_GetTextureWidth(
	S3tTestInfoP		testInfo);

long S3TestInfo_GetTextureHeight(
	S3tTestInfoP		testInfo);

long S3TestInfo_GetTextureRowBytes(
	S3tTestInfoP		testInfo);

/*
	Texture flags
*/
unsigned long S3TestInfo_GetTextureFlags(
	S3tTestInfoP		testInfo);

TQAImagePixelType S3TestInfo_GetTextureFormat(
	S3tTestInfoP		testInfo);
	
Boolean S3TestInfo_GetDetachTexture(
	S3tTestInfoP		testInfo);
	
Boolean S3TestInfo_GetTextureMakeBlackTransparent(
	S3tTestInfoP		testInfo);

/*
	Texture color table
*/
TQAColorTable* S3TestInfo_GetLTextureColorTable(
	S3tTestInfoP		testInfo);
	
TQAColorTable* S3TestInfo_GetRTextureColorTable(
	S3tTestInfoP		testInfo);
	
TQAColorTableType S3TestInfo_GetTextureColorTableType(
	S3tTestInfoP		testInfo);
	
Boolean S3TestInfo_GetTextureUseTransparentIndex(
	S3tTestInfoP		testInfo);
	
/*
	Bitmap Image
*/
S3Image S3TestInfo_GetBitmapImage(
	S3tTestInfoP		testInfo);

/*
	L, R Bitmap
*/
TQABitmap *S3TestInfo_GetLBitmap(
	S3tTestInfoP		testInfo);

TQABitmap *S3TestInfo_GetRBitmap(
	S3tTestInfoP		testInfo);

/*
	Bitmap Size
*/
long S3TestInfo_GetBitmapWidth(
	S3tTestInfoP		testInfo);
	
long S3TestInfo_GetBitmapHeight(
	S3tTestInfoP		testInfo);
	
long S3TestInfo_GetBitmapRowBytes(
	S3tTestInfoP		testInfo);
	
/*
	Bitmap flags
*/
unsigned long S3TestInfo_GetBitmapFlags(
	S3tTestInfoP		testInfo);

TQAImagePixelType S3TestInfo_GetBitmapFormat(
	S3tTestInfoP		testInfo);
	
Boolean S3TestInfo_GetDetachBitmap(
	S3tTestInfoP		testInfo);
	
Boolean S3TestInfo_GetBitmapMakeBlackTransparent(
	S3tTestInfoP		testInfo);
	
/*
	Bitmap color table
*/
TQAColorTable* S3TestInfo_GetLBitmapColorTable(
	S3tTestInfoP		testInfo);
	
TQAColorTable* S3TestInfo_GetRBitmapColorTable(
	S3tTestInfoP		testInfo);
	
TQAColorTableType S3TestInfo_GetBitmapColorTableType(
	S3tTestInfoP		testInfo);
	
Boolean S3TestInfo_GetBitmapUseTransparentIndex(
	S3tTestInfoP		testInfo);
	
/*
	Clip
*/
S3eClipRegion S3TestInfo_GetClipMenu(
	S3tTestInfoP		testInfo);


#ifdef __cplusplus
}
#endif



