/*****************************************************************************\

	S3EvenMoreTests.c

	Copyright (C) 1995 Apple Computer, Inc.  All rights reserved.

\*****************************************************************************/

/*===========================================================================*\
	Includes
\*===========================================================================*/
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <Files.h>
#include <Memory.h>
#include <Quickdraw.h>

#include "Drive3D.h"
#include "S3Image.h"
#include "S3TestSystem.h"
#include "S3TestInfo.h"
#include "S3Utilities.h"

/*===========================================================================*\
	Defines
\*===========================================================================*/
#define kPi	 		((float)3.1415926535898)

/*===========================================================================*\
	Prototypes
\*===========================================================================*/

void S3EvenMoreTests_Register(void);
S3tTestStatus S3IntersectTrisZAnimate(S3tTestInfo *testInfo);
S3tTestStatus S3LineFan(S3tTestInfo *testInfo);
S3tTestStatus S3LineZTest(S3tTestInfo *testInfo);

/*===========================================================================*\
	S3EvenMoreTests_Register
\*===========================================================================*/
void S3EvenMoreTests_Register(void)
{
	S3Test_Register("Line Fan",						kS3Prereq_None, S3LineFan);
	S3Test_Register("Animated Z Gouraud Tris",		kS3Prereq_None, S3IntersectTrisZAnimate);
	S3Test_Register("Line Z Test",					kS3Prereq_None, S3LineZTest);
}

/*===========================================================================*\
	S3IntersectTris
\*===========================================================================*/
S3tTestStatus S3IntersectTrisZAnimate(
	S3tTestInfo		*testInfo)
{
	float xmax = (float) (S3TestInfo_GetHeight(testInfo) - 0.1) / 2.0;
	float ymax = (float) (S3TestInfo_GetWidth(testInfo) - 0.1) / 2.0;

	float xmid = xmax / 2.0;
	float ymid = ymax / 2.0;
	float ytwothirds = ymax * 2.0 / 3.0;
	
	TQAVGouraud vertices[6] = {
		{ 0.0, 0.0, 0.5, 1.0 / 0.5,
		  1.0, 0.0, 0.0, 1.0 },
		{ 0.0, 0.0, 0.5, 1.0 / 0.5,
		  1.0, 0.0, 0.0, 1.0 },
		{ 0.0, 0.0, 0.1, 1.0 / 0.1,
		  1.0, 0.0, 0.0, 1.0 },

		{ 0.0, 0.0, 0.1, 1.0 / 0.1,
		  0.0, 1.0, 0.0, 1.0 },
		{ 0.0, 0.0, 0.3, 1.0 / 0.3,
		  0.0, 1.0, 0.0, 1.0 },
		{ 0.0, 0.0, 0.3, 1.0 / 0.3,
		  0.0, 1.0, 0.0, 1.0 }
	};
	long	i, j;
	float	sign;
	
	for (j = 0; j < 20; j++)
	{
		float	stepIn	= (float) j / 20.0;
		float	stepOut = 1.0 - stepIn;
		
		vertices[0].z = stepOut;
		vertices[1].z = stepOut;
		vertices[2].z = stepIn;
		
		vertices[3].z = stepOut;
		vertices[4].z = stepOut;
		vertices[5].z = stepIn;
		
		for (i = 0; i < 6; i++)
		{
			vertices[i].invW	= 1.0 / vertices[i].z;
			vertices[i].r		= (i < 3) ? 1.0 : 0.0;
			vertices[i].g		= 1.0 - vertices[i].r;
			vertices[i].b		= 0.0;
			vertices[i].a		= 1.0;
		}
		
		S3Test_RenderStart(testInfo, NULL);
		
		for (i = 0; i < 4; i++)
		{
			sign = (i & 1) ? 1 : -1;
	
			vertices[0].x = xmax + sign * xmax; 
			vertices[1].x = xmax + sign * xmax; 
			vertices[2].x = xmax + sign * 0.0; 
			
			vertices[3].x = xmax + sign * xmid; 
			vertices[4].x = xmax + sign * 0.0; 
			vertices[5].x = xmax + sign * xmax; 
	
			sign = (i & 2) ? 1 : -1;
	
			vertices[0].y = ymax + sign * ymax; 
			vertices[1].y = ymax + sign * 0.0; 
			vertices[2].y = ymax + sign * ymid; 
	
			vertices[3].y = ymax + sign * ymax; 
			vertices[4].y = ymax + sign * 0.0; 
			vertices[5].y = ymax + sign * ytwothirds; 
	
			S3Test_DrawTriGouraud(testInfo, &vertices[0], &vertices[1], &vertices[2], kQATriFlags_None);
			S3Test_DrawTriGouraud(testInfo, &vertices[3], &vertices[4], &vertices[5], kQATriFlags_None);
		}
			
		S3Test_RenderEnd(testInfo, NULL);
	}
		
	return kS3Status_Completed;
}

/*===========================================================================*\
	S3LineFan
\*===========================================================================*/
S3tTestStatus S3LineFan(
	S3tTestInfo *testInfo)
{
	float			xCenter, yCenter;
	float			xEnd, yEnd;
	float			r, rEnd, dr, rScale, temp;
	TQAVGouraud		v0, v1;
	long			colorChannel;
	
	xCenter = (float) S3TestInfo_GetHeight(testInfo) / 2.0;
	yCenter = (float) S3TestInfo_GetWidth(testInfo) / 2.0;
	
	rEnd	= (kPi * 2.0);
	dr		= rEnd / 24.0;
	rScale	= 0.9F;

	//((xCenter > yCenter) ? yCenter : xCenter) * 0.1 / 12.0));

	v0.x	= xCenter;
	v0.y	= yCenter;
	v0.z	= 0.1F;
	v0.invW	= 1.0F / v0.z;
	v0.a	= 1.0F;
	v0.r	= 1.0F;
	v0.g	= 1.0F;
	v0.b	= 1.0F;
	
	v1.z	= 0.9F;
	v1.invW	= 1.0F / v1.z;
	v1.a	= 1.0F;

	S3Test_RenderStart(testInfo, NULL);
	for (r = 0.0F; r < rEnd; r += dr)
	{
		xEnd = (cos((double) r) * rScale + 1.0F) * xCenter;
		yEnd = ((temp = sin((double) r)) * rScale + 1.0F) * yCenter;

		colorChannel = (long) (r * 2.98 / rEnd);
		ASSERT((colorChannel >= 0) && (colorChannel <= 2));
		
		v1.x	= xEnd;
		v1.y	= yEnd;

		v1.r	= 1.0F;
		v1.g	= 1.0F;
		v1.b	= 1.0F;
		(&v1.r)[colorChannel] = fabs(temp);
		
		S3Test_DrawLine(testInfo, &v0, &v1);
	}
	S3Test_RenderEnd(testInfo, NULL);
	return kS3Status_Completed;
}

/*===========================================================================*\
	S3LineZTest
\*===========================================================================*/
S3tTestStatus S3LineZTest(
	S3tTestInfo *testInfo)
{
	(void) testInfo;
	
	return kS3Status_Completed;
}

