/*****************************************************************************\

	S3RandomTests.c

	Random Points, Lines, TriGouraud, TriTexture, TriMesh
	
	Copyright (C) 1995 Apple Computer, Inc.  All rights reserved.

\*****************************************************************************/

/*=========================================================================*\
	Includes
\*=========================================================================*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "S3TestSystem.h"
#include "S3TestInfo.h"
#include "S3TestUtil.h"


/*=========================================================================*\
	Constants
\*=========================================================================*/

#define kNPoints					10000
#define kNLines						1000
#define kNTris						100
#define kNFlushes					5


/*=========================================================================*\
	Prototypes
\*=========================================================================*/

void S3RandomTests_Register(void);
S3tTestStatus S3TestPoints(
	S3tTestInfo				*testInfo);
S3tTestStatus S3TestLines(
	S3tTestInfo				*testInfo);
S3tTestStatus S3TestTriGouraud(
	S3tTestInfo				*testInfo);
S3tTestStatus S3TestTriGouraudAlpha(
	S3tTestInfo				*testInfo);
S3tTestStatus S3TestTriMeshGouraud(
	S3tTestInfo				*testInfo);
S3tTestStatus S3TestTriTexture(
	S3tTestInfo				*testInfo);
S3tTestStatus S3TestTriTextureAlpha(
	S3tTestInfo				*testInfo);
S3tTestStatus S3TestTriMeshTexture(
	S3tTestInfo				*testInfo);

	
/*=========================================================================*\
	S3RandomTests_Register
\*=========================================================================*/
void S3RandomTests_Register(void)
{
	S3Test_Register("Random Gouraud Tris",			kS3Prereq_None, S3TestTriGouraud);
	S3Test_Register("Random Gouraud Alpha Tris",	kS3Prereq_None, S3TestTriGouraudAlpha);
	S3Test_Register("Random Gouraud TriMesh",		kS3Prereq_None, S3TestTriMeshGouraud);
	S3Test_Register("Random Texture Tris",			kS3Prereq_Texture, S3TestTriTexture);
	S3Test_Register("Random Texture Alpha Tris",	kS3Prereq_Texture, S3TestTriTextureAlpha);
	S3Test_Register("Random Texture TriMesh",		kS3Prereq_Texture, S3TestTriMeshTexture);
	S3Test_Register("Random Lines",					kS3Prereq_None, S3TestLines);
	S3Test_Register("Random Points",				kS3Prereq_None, S3TestPoints);
}

/*=========================================================================*\
	S3TestPoints
\*=========================================================================*/
S3tTestStatus S3TestPoints(
	S3tTestInfo*			inTestInfo)
{
	long					i, flushCount;
	TQAVGouraud				vGouraud;
	long					pointCount = kNPoints;
	char* 					argStringP = S3TestInfo_GetArgString(inTestInfo);
	S3tTestStatus			status = kS3Status_Completed;

		// scan the argument string to get the number of points to draw
	sscanf(argStringP, "%lu", &pointCount);

	S3Test_Print("Number of points: %d\r", pointCount);
	
	S3Test_RenderStart(inTestInfo, NULL);

	for (i = 0, flushCount = pointCount / kNFlushes; i++ < pointCount; /* Nothing */)
	{
		S3TestUtil_RandomVGouraud(inTestInfo, &vGouraud);
		
		S3Test_DrawPoint(inTestInfo, &vGouraud);

		if ((i % flushCount) == 0)
		{
			S3Test_Flush(inTestInfo);
			
			if (S3TestInfo_CheckForCancel(inTestInfo)) {
				status = kS3Status_Cancelled;
				break;
			}
		}
	}

	S3Test_RenderEnd(inTestInfo, NULL);
	
	return status;
}

/*=========================================================================*\
	S3TestLines
\*=========================================================================*/
S3tTestStatus S3TestLines(
	S3tTestInfo*			inTestInfo)
{
	long					i, flushCount;
	TQAVGouraud				vGouraud[2];
	long					lineCount = kNLines;
	char* 					argStringP = S3TestInfo_GetArgString(inTestInfo);
	S3tTestStatus			status = kS3Status_Completed;

		// scan the argument string to get the number of lines to draw
	sscanf(argStringP, "%lu", &lineCount);
		
	S3Test_Print("Number of lines: %d\r", lineCount);
	
	S3Test_RenderStart(inTestInfo, NULL);

	for (i = 0, flushCount = lineCount / kNFlushes; i++ < lineCount; /* Nothing */)
	{
		S3TestUtil_RandomVGouraud(inTestInfo, &vGouraud[0]);
		S3TestUtil_RandomVGouraud(inTestInfo, &vGouraud[1]);

		S3Test_DrawLine(inTestInfo, &vGouraud[0], &vGouraud[1]);
		
		if ((i % flushCount) == 0)
		{
			S3Test_Flush(inTestInfo);
			
			if (S3TestInfo_CheckForCancel(inTestInfo)) {
				status = kS3Status_Cancelled;
				break;
			}
		}
	}

	S3Test_RenderEnd(inTestInfo, NULL);

	return status;
}
					
/*=========================================================================*\
	S3TestTriGouraud
\*=========================================================================*/
S3tTestStatus S3TestTriGouraud(
	S3tTestInfo*			inTestInfo)
{
	long					i, flushCount;
	long					triangleCount = kNTris;
	char* 					argStringP = S3TestInfo_GetArgString(inTestInfo);
	S3tTestStatus			status = kS3Status_Completed;

		// scan the argument string to get the number of tris to draw
	sscanf(argStringP, "%lu", &triangleCount);
		
	S3Test_Print("Number of triangles: %d\r", triangleCount);
	
	S3Test_RenderStart(inTestInfo, NULL);

	for (i = 0, flushCount = triangleCount / kNFlushes; i++ < triangleCount; /* Nothing */)
	{
		S3TestUtil_DrawRandomTriGouraud(inTestInfo);
		
		if ((i % flushCount) == 0)
		{
			S3Test_Flush(inTestInfo);
			
			if (S3TestInfo_CheckForCancel(inTestInfo)) {
				status = kS3Status_Cancelled;
				break;
			}
		}
	}

	S3Test_RenderEnd(inTestInfo, NULL);

	return status;
}
					
/*=========================================================================*\
	S3TestTriGouraudAlpha
\*=========================================================================*/
S3tTestStatus S3TestTriGouraudAlpha(
	S3tTestInfo*			inTestInfo)
{
	TQAVGouraud				v[3];
	long					i, flushCount;
	long					triangleCount = kNTris;
	char* 					argStringP = S3TestInfo_GetArgString(inTestInfo);
	S3tTestStatus			status = kS3Status_Completed;

		// scan the argument string to get the number of tris to draw
	sscanf(argStringP, "%lu", &triangleCount);
		
	S3Test_Print("Number of triangles: %d\r", triangleCount);
	
	S3Test_RenderStart(inTestInfo, NULL);

	for (i = 0, flushCount = triangleCount / kNFlushes; i++ < triangleCount; /* Nothing */)
	{
		S3TestUtil_RandomAlphaVGouraud(inTestInfo, &v[0]);
		S3TestUtil_RandomAlphaVGouraud(inTestInfo, &v[1]);
		S3TestUtil_RandomAlphaVGouraud(inTestInfo, &v[2]);
		
		S3Test_DrawTriGouraud(inTestInfo, &v[0], &v[1], &v[2], kQATriFlags_None);
		
		if ((i % flushCount) == 0)
		{
			S3Test_Flush(inTestInfo);
			
			if (S3TestInfo_CheckForCancel(inTestInfo)) {
				status = kS3Status_Cancelled;
				break;
			}
		}
	}

	S3Test_RenderEnd(inTestInfo, NULL);

	return status;
}
					
/*=========================================================================*\
	S3TestTriMeshGouraud
\*=========================================================================*/
S3tTestStatus S3TestTriMeshGouraud(
	S3tTestInfo*			inTestInfo)
{
	TQAVGouraud*			vertices = nil;
	TQAIndexedTriangle*		triangles = nil;
	long					i;
	long					triangleCount = kNTris;
	long					vertexCount = kNTris * 2;
	char* 					argStringP = S3TestInfo_GetArgString(inTestInfo);
	S3tTestStatus			status = kS3Status_Completed;

		// scan the argument string to get the number of tris to draw
	sscanf(argStringP, "%lu", &triangleCount);
		
	S3Test_Print("Number of triangles: %d\r", triangleCount);
	
		// allocate 2 vertices for every triangle the user wants us to draw.
		// this means that on average every triangle will be connected to at
		// least one other.
	vertices = malloc(2 * triangleCount * sizeof(TQAVGouraud));
	
	if (vertices == nil) {
		S3Test_Print("*** There is not enough memory to run this test. Try  ***\r"
					 "*** passing in a smaller number of triangles to draw. ***\r");
		
		status = kS3Status_Failure;
		goto cleanup;
	}
	
		// allocate a single TQAIndexedTriangle for every triangle the user
		// wants us to draw.  these will hold the indexes into the vertices.
	triangles = malloc(triangleCount * sizeof(TQAIndexedTriangle));
	
	if (triangles == nil) {
		S3Test_Print("*** There is not enough memory to run this test. Try  ***\r"
					 "*** passing in a smaller number of triangles to draw. ***\r");
		
		status = kS3Status_Failure;
		goto cleanup;
	}
	
		// fill in the array of vertices with random values
	for (i = 0; i < vertexCount; i++) {
		S3TestUtil_RandomVGouraud(inTestInfo, &vertices[i]);
	} // endfor
	
		// fill in the default tri flags for each triangle, and calculate a 
		// random index for each of the three vertex indexes.
	for (i = 0; i < triangleCount; i++) {
		triangles[i].triangleFlags = kQATriFlags_None;
		
		triangles[i].vertices[0] = 
				(long) (S3TestUtil_RandomFloat() * (vertexCount - 1));
		triangles[i].vertices[1] = 
				(long) (S3TestUtil_RandomFloat() * (vertexCount - 1));
		triangles[i].vertices[2] = 
				(long) (S3TestUtil_RandomFloat() * (vertexCount - 1));
	} // endfor

	S3Test_RenderStart(inTestInfo, NULL);

		S3Test_SubmitVerticesGouraud(inTestInfo, vertexCount, vertices);
		
		S3Test_DrawTriMeshGouraud(inTestInfo, triangleCount, triangles);
		
	S3Test_RenderEnd(inTestInfo, NULL);

cleanup:
	if (vertices != nil) {
		free(vertices);
	}
	
	if (triangles != nil) {
		free(triangles);
	}
	
	return status;
}

/*=========================================================================*\
	S3TestTriTexture
\*=========================================================================*/
S3tTestStatus S3TestTriTexture(
	S3tTestInfo*			inTestInfo)
{
	long					i, flushCount;
	long					triangleCount = kNTris;
	char* 					argStringP = S3TestInfo_GetArgString(inTestInfo);
	S3tTestStatus			status = kS3Status_Completed;

		// scan the argument string to get the number of tris to draw
	sscanf(argStringP, "%lu", &triangleCount);
		
	S3Test_Print("Number of triangles: %d\r", triangleCount);
	
	S3Test_RenderStart(inTestInfo, NULL);

	for (i = 0, flushCount = triangleCount / kNFlushes; i++ < triangleCount; /* Nothing */)
	{
		S3TestUtil_DrawRandomTriTexture(inTestInfo);
		
		if ((i % flushCount) == 0)
		{
			S3Test_Flush(inTestInfo);
			
			if (S3TestInfo_CheckForCancel(inTestInfo)) {
				status = kS3Status_Cancelled;
				break;
			}
		}
	}

	S3Test_RenderEnd(inTestInfo, NULL);

	return status;
}			
					
/*=========================================================================*\
	S3TestTriTextureAlpha
\*=========================================================================*/
S3tTestStatus S3TestTriTextureAlpha(
	S3tTestInfo*			inTestInfo)
{
	TQAVTexture				v[3];
	long					i, flushCount;
	long					triangleCount = kNTris;
	char* 					argStringP = S3TestInfo_GetArgString(inTestInfo);
	S3tTestStatus			status = kS3Status_Completed;

		// scan the argument string to get the number of tris to draw
	sscanf(argStringP, "%lu", &triangleCount);
		
	S3Test_Print("Number of triangles: %d\r", triangleCount);
	
	S3Test_RenderStart(inTestInfo, NULL);

	for (i = 0, flushCount = triangleCount / kNFlushes; i++ < triangleCount; /* Nothing */)
	{
		v[0].uOverW = 0.0;
		v[0].vOverW = 0.0;
	
		v[1].uOverW = 0.0;
		v[1].vOverW = 1.0;
	
		v[2].uOverW = 1.0;
		v[2].vOverW = 1.0;
	
		S3TestUtil_RandomAlphaVTexture(inTestInfo, &v[0]);
		S3TestUtil_RandomAlphaVTexture(inTestInfo, &v[1]);
		S3TestUtil_RandomAlphaVTexture(inTestInfo, &v[2]);
		
		S3Test_DrawTriTexture(inTestInfo, &v[0], &v[1], &v[2], kQATriFlags_None);
		
		if ((i % flushCount) == 0)
		{
			S3Test_Flush(inTestInfo);
			
			if (S3TestInfo_CheckForCancel(inTestInfo)) {
				status = kS3Status_Cancelled;
				break;
			}
		}
	}

	S3Test_RenderEnd(inTestInfo, NULL);

	return status;
}
					
/*=========================================================================*\
	S3TestTriMeshTexture
\*=========================================================================*/
S3tTestStatus S3TestTriMeshTexture(
	S3tTestInfo*			inTestInfo)
{
	TQAVTexture*			vertices = nil;
	TQAIndexedTriangle*		triangles = nil;
	S3tTestStatus			status = kS3Status_Completed;
	long					i;
	long					triangleCount = kNTris;
	long					vertexCount = kNTris * 2;
	char* 					argStringP = S3TestInfo_GetArgString(inTestInfo);

		// scan the argument string to get the number of tris to draw
	sscanf(argStringP, "%lu", &triangleCount);
		
	S3Test_Print("Number of triangles: %d\r", triangleCount);
	
		// allocate 2 vertices for every triangle the user wants us to draw.
		// this means that on average every triangle will be connected to at
		// least one other.
	vertices = malloc(2 * triangleCount * sizeof(TQAVTexture));
	
	if (vertices == nil) {
		S3Test_Print("*** There is not enough memory to run this test. Try  ***\r"
					 "*** passing in a smaller number of triangles to draw. ***\r");
		
		status = kS3Status_Failure;
		goto cleanup;
	}
	
		// allocate a single TQAIndexedTriangle for every triangle the user
		// wants us to draw.  these will hold the indexes into the vertices.
	triangles = malloc(triangleCount * sizeof(TQAIndexedTriangle));
	
	if (triangles == nil) {
		S3Test_Print("*** There is not enough memory to run this test. Try  ***\r"
					 "*** passing in a smaller number of triangles to draw. ***\r");
		
		status = kS3Status_Failure;
		goto cleanup;
	}
	
		// fill in the array of vertices with random values.  we have to fill
		// in the uOverW and vOverW values before calling RandomVTexture because 
		// that function assumes they've already been set.
	for (i = 0; i < vertexCount; i++) {
		vertices[i].uOverW = S3TestUtil_RandomFloat();
		vertices[i].vOverW = S3TestUtil_RandomFloat();
		
		S3TestUtil_RandomVTexture(inTestInfo, &vertices[i]);
	} // endfor
	
		// fill in the default tri flags for each triangle, and calculate a 
		// random index for each of the three vertex indexes.
	for (i = 0; i < triangleCount; i++) {
		triangles[i].triangleFlags = kQATriFlags_None;
		
		triangles[i].vertices[0] = 
				(long) (S3TestUtil_RandomFloat() * (vertexCount - 1));
		triangles[i].vertices[1] = 
				(long) (S3TestUtil_RandomFloat() * (vertexCount - 1));
		triangles[i].vertices[2] = 
				(long) (S3TestUtil_RandomFloat() * (vertexCount - 1));
	} // endfor

	S3Test_RenderStart(inTestInfo, NULL);

		S3Test_SubmitVerticesTexture(inTestInfo, vertexCount, vertices);
		
		S3Test_DrawTriMeshTexture(inTestInfo, triangleCount, triangles);
		
	S3Test_RenderEnd(inTestInfo, NULL);

cleanup:
	if (vertices != nil) {
		free(vertices);
	}
	
	if (triangles != nil) {
		free(triangles);
	}
	
	return status;
}

